"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.wrappers_pb2
import tensorflow.compiler.xla.service.hlo_pb2

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class ClippingLimits(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LOWER_FIELD_NUMBER: builtins.int
    UPPER_FIELD_NUMBER: builtins.int
    @property
    def lower(self) -> google.protobuf.wrappers_pb2.FloatValue:
        """-inf if not set"""

    @property
    def upper(self) -> google.protobuf.wrappers_pb2.FloatValue:
        """+inf if not set"""

    def __init__(
        self,
        *,
        lower: google.protobuf.wrappers_pb2.FloatValue | None = ...,
        upper: google.protobuf.wrappers_pb2.FloatValue | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["lower", b"lower", "upper", b"upper"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["lower", b"lower", "upper", b"upper"]) -> None: ...

global___ClippingLimits = ClippingLimits

@typing.final
class SimulatedQuantization(google.protobuf.message.Message):
    """Configuration for simulated quantization; simulated quantization is used to
    reduce training/serving skew when the serving variables are quantized. The
    same quantization operations are executed during training to minimize
    differences with serving.

    Simulated quantization inserts the following operations on the forward pass
    after gathering the embedding vector from HBM. The backward pass operations
    are unchanged.

    clipped_val = clip(input, clipping_limits)
    quantum = clipping_limits.range() / (num_buckets - 1)
    quantized_val = floor((clipped_val - clipping_limits.lower()) / quantum + .5)
    return quantized_val * quantum + clipping_limits.lower().
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ENABLED_FIELD_NUMBER: builtins.int
    CLIPPING_LIMITS_FIELD_NUMBER: builtins.int
    NUM_BUCKETS_FIELD_NUMBER: builtins.int
    enabled: builtins.bool
    """Whether simulated quantization is enabled."""
    num_buckets: builtins.int
    """Number of possible quantized values."""
    @property
    def clipping_limits(self) -> global___ClippingLimits:
        """Minimum and maximum values of the range used for quantization."""

    def __init__(
        self,
        *,
        enabled: builtins.bool | None = ...,
        clipping_limits: global___ClippingLimits | None = ...,
        num_buckets: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["clipping_limits", b"clipping_limits"]) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal["clipping_limits", b"clipping_limits", "enabled", b"enabled", "num_buckets", b"num_buckets"],
    ) -> None: ...

global___SimulatedQuantization = SimulatedQuantization

@typing.final
class DynamicLearningRate(google.protobuf.message.Message):
    """Dynamic learning rate specification in the TPUEmbeddingConfiguration. The
    actual learning rates are provided as a scalar input list to the
    SendTPUEmbeddingGradients Op indexed by their tag specified through the
    following proto.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TAG_FIELD_NUMBER: builtins.int
    tag: builtins.int
    """For tables where learning rates are dynamically computed and communicated
    to the TPU embedding program, a tag must be specified for the learning
    rate.

    The tag must be a non-negative  integer. The total number of unique tags
    must be less than or equal to the number of tables in the TPU embedding
    configuration (a table does not specify any tag if it uses a constant
    learning rate, and specifies exactly one tag if it uses dynamic learning
    rates).

    All tags in the range [0, number_of_unique_tags) must be present in the TPU
    embedding configuration, i.e. a tag cannot be skipped if a different tag
    numerically greater than it is used in the configuration.

    If multiple tables specify the same tag, they *MUST* have
    the same dynamic learning rate, for example, their dynamic learning rate
    could be computed by the same TensorFlow sub-graph. The partitioning of the
    embedding layer would be more optimal if the number_of_unique_tags is as
    *LOW* as possible, i.e., if many tables share the same tag.

    The learning_rate input of the SendTPUEmbeddingGradients op is used to
    communicate dynamic learning rates to the TPU embedding program.
    The learning_rate input is a list of scalars where the size of the list is
    equal to the number of unique tags. The learning rate associated with a
    particular tag is specified by populating its corresponding index in the
    list of learning_rate scalars.
    """
    def __init__(self, *, tag: builtins.int | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["tag", b"tag"]) -> None: ...

global___DynamicLearningRate = DynamicLearningRate

@typing.final
class LearningRate(google.protobuf.message.Message):
    """Source of learning rate to use."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CONSTANT_FIELD_NUMBER: builtins.int
    DYNAMIC_FIELD_NUMBER: builtins.int
    constant: builtins.float
    @property
    def dynamic(self) -> global___DynamicLearningRate: ...
    def __init__(self, *, constant: builtins.float | None = ..., dynamic: global___DynamicLearningRate | None = ...) -> None: ...
    def HasField(
        self, field_name: typing.Literal["constant", b"constant", "dynamic", b"dynamic", "learning_rate", b"learning_rate"]
    ) -> builtins.bool: ...
    def ClearField(
        self, field_name: typing.Literal["constant", b"constant", "dynamic", b"dynamic", "learning_rate", b"learning_rate"]
    ) -> None: ...
    def WhichOneof(
        self, oneof_group: typing.Literal["learning_rate", b"learning_rate"]
    ) -> typing.Literal["constant", "dynamic"] | None: ...

global___LearningRate = LearningRate

@typing.final
class AdagradParameters(google.protobuf.message.Message):
    """Each optimizer's parameter proto has a link to its documentation and CPU
    implementation (if available) for user reference.

    https://www.tensorflow.org/api_docs/python/tf/keras/optimizers/Adagrad
    https://github.com/tensorflow/tensorflow/blob/6b6471f3ffb7f1fefe42d814aa5fb9ab7a535b58/tensorflow/core/kernels/training_ops.cc#L1634
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(self) -> None: ...

global___AdagradParameters = AdagradParameters

@typing.final
class AdagradMomentumParameters(google.protobuf.message.Message):
    """This optimizer combines the Adagrad and Momentum update rules.
    accum(new) = beta2 == 1.0 ?
                 accum(old) + grad^2 :
                 beta2 * accum(old) + (1 - beta2) * grad^2
    accum_with_exponent = (accum(new) + epsilon)^(-1.0 / exponent)
    mom_accum(new) = momentum * mom_accum(old) + accum_with_exponent
    update = use_nesterov ?
             momentum * mom_accum(new) + accum_with_exponent :
             mom_accum(new)
    var(new) = var(old) - lr * grad * update
    Algorithm described in https://arxiv.org/abs/2002.11803.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MOMENTUM_FIELD_NUMBER: builtins.int
    USE_NESTEROV_FIELD_NUMBER: builtins.int
    EXPONENT_FIELD_NUMBER: builtins.int
    BETA2_FIELD_NUMBER: builtins.int
    EPSILON_FIELD_NUMBER: builtins.int
    momentum: builtins.float
    """Moving average parameter for the momentum accumulator."""
    use_nesterov: builtins.bool
    """Whether to use the Nesterov variant of momentum."""
    exponent: builtins.float
    """Exponent for the gradient^2 accumulator."""
    beta2: builtins.float
    """Moving average parameter for the gradient^2 accumulator."""
    epsilon: builtins.float
    """Offset added to the Adagrad accumulator."""
    def __init__(
        self,
        *,
        momentum: builtins.float | None = ...,
        use_nesterov: builtins.bool | None = ...,
        exponent: builtins.float | None = ...,
        beta2: builtins.float | None = ...,
        epsilon: builtins.float | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "beta2",
            b"beta2",
            "epsilon",
            b"epsilon",
            "exponent",
            b"exponent",
            "momentum",
            b"momentum",
            "use_nesterov",
            b"use_nesterov",
        ],
    ) -> None: ...

global___AdagradMomentumParameters = AdagradMomentumParameters

@typing.final
class BoundedAdagradParameters(google.protobuf.message.Message):
    """Algorithm in http://www.jmlr.org/papers/volume12/duchi11a/duchi11a.pdf."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    UPDATE_ACCUMULATOR_FIRST_FIELD_NUMBER: builtins.int
    MAX_VAR_UPDATE_FIELD_NUMBER: builtins.int
    MAX_ACCUMULATOR_FIELD_NUMBER: builtins.int
    update_accumulator_first: builtins.bool
    """Whether to use the updated or the old value of the accumulator when
    computing the effective learning rate. When update_accumulator_first is set
    to True, the updated value of the accumulator is used.
    """
    max_var_update: builtins.float
    """The max_var_update value to use. Set value to 0 (default) to disable using
    max_var_update to clip the gradient.
    """
    max_accumulator: builtins.float
    """The maximum value of the accumulator. Set max_accumulator to 0 (default)
    to disable using max_accumulator to clip the accumulator.
    """
    def __init__(
        self,
        *,
        update_accumulator_first: builtins.bool | None = ...,
        max_var_update: builtins.float | None = ...,
        max_accumulator: builtins.float | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "max_accumulator",
            b"max_accumulator",
            "max_var_update",
            b"max_var_update",
            "update_accumulator_first",
            b"update_accumulator_first",
        ],
    ) -> None: ...

global___BoundedAdagradParameters = BoundedAdagradParameters

@typing.final
class StochasticGradientDescentParameters(google.protobuf.message.Message):
    """https://www.tensorflow.org/api_docs/python/tf/keras/optimizers/SGD
    https://github.com/tensorflow/tensorflow/blob/6b6471f3ffb7f1fefe42d814aa5fb9ab7a535b58/tensorflow/core/kernels/training_ops.cc#L629
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(self) -> None: ...

global___StochasticGradientDescentParameters = StochasticGradientDescentParameters

@typing.final
class FtrlParameters(google.protobuf.message.Message):
    """https://www.tensorflow.org/api_docs/python/tf/keras/optimizers/Ftrl
    https://static.googleusercontent.com/media/research.google.com/en//pubs/archive/41159.pdf
    https://github.com/tensorflow/tensorflow/blob/6b6471f3ffb7f1fefe42d814aa5fb9ab7a535b58/tensorflow/core/kernels/training_ops.cc#L2646

    The hyperparameters for FTRL are the same as for the Keras implementation,
    with some additions. The "beta" parameter matches the behavior described in
    the second link above; "beta" / (2 * learning rate) should be added to "l2"
    to get equivalent behavior in the other TensorFlow implementations of this
    optimizer. When the multiply_linear_by_lr field is set to true, a modified
    formula is used for FTRL that treats the "linear" accumulator as being
    pre-multiplied by the learning rate (i.e., the accumulator named "linear"
    actually stores "linear * learning_rate"). Other than checkpoint
    compatibility, this is mathematically equivalent for a static learning rate;
    for a dynamic learning rate, it is nearly the same as long as the learning
    rate does not change quickly. The benefit of setting multiply_linear_by_lr to
    true is that the modified formula handles zero and near-zero learning rates
    without producing NaNs, improving flexibility for learning rate ramp-up.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    L1_FIELD_NUMBER: builtins.int
    L2_FIELD_NUMBER: builtins.int
    LR_POWER_FIELD_NUMBER: builtins.int
    BETA_FIELD_NUMBER: builtins.int
    MULTIPLY_LINEAR_BY_LR_FIELD_NUMBER: builtins.int
    ALLOW_ZERO_ACCUMULATOR_FIELD_NUMBER: builtins.int
    l1: builtins.float
    l2: builtins.float
    lr_power: builtins.float
    beta: builtins.float
    multiply_linear_by_lr: builtins.bool
    allow_zero_accumulator: builtins.bool
    """Previously, allow_zero_accumulator parameter changed some internal formulas
    to allow zero and near-zero accumulator values at the cost of some
    performance. The current implementation ignores this parameter; zero or
    near-zero accumulator values are now always supported.
    """
    def __init__(
        self,
        *,
        l1: builtins.float | None = ...,
        l2: builtins.float | None = ...,
        lr_power: builtins.float | None = ...,
        beta: builtins.float | None = ...,
        multiply_linear_by_lr: builtins.bool | None = ...,
        allow_zero_accumulator: builtins.bool | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "allow_zero_accumulator",
            b"allow_zero_accumulator",
            "beta",
            b"beta",
            "l1",
            b"l1",
            "l2",
            b"l2",
            "lr_power",
            b"lr_power",
            "multiply_linear_by_lr",
            b"multiply_linear_by_lr",
        ],
    ) -> None: ...

global___FtrlParameters = FtrlParameters

@typing.final
class AdamParameters(google.protobuf.message.Message):
    """The Adam optimizer does not implement hyper-parameter update due to hardware
    limitations; use the dynamic learning rate feature instead, setting the
    learning rate to: user learning_rate * sqrt(1 - beta2^t) / (1 - beta1^t)
    Here, t is the current timestep.

    https://www.tensorflow.org/api_docs/python/tf/keras/optimizers/Adam
    https://github.com/tensorflow/tensorflow/blob/ab51450c817674c8ff08a7ae4f8ac50cdc4bed8b/tensorflow/python/training/adam.py#L32

    Note that the code by default implements the lazy version of Adam
    (https://www.tensorflow.org/api_docs/python/tf/contrib/opt/LazyAdamOptimizer)
    unless the use_non_lazy_adam parameter is set, in which case it implements
    the normal version of Adam that updates all parameters in the embedding
    table, even for entries that are not used in the current minibatch
    (https://www.tensorflow.org/api_docs/python/tf/contrib/opt/AdamOptimizer). If
    use_non_lazy_adam is enabled, gradient accumulation is also required to be
    enabled in order to get correct results; a warning will be printed otherwise
    (which may change to an error in the future). If use_sum_inside_sqrt is set,
    the Adam variable update formula will be changed from m / (sqrt(v) + epsilon)
    to m / sqrt(v + epsilon**2); this option improves the performance of TPU
    training and is not expected to harm model quality.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BETA1_FIELD_NUMBER: builtins.int
    BETA2_FIELD_NUMBER: builtins.int
    EPSILON_FIELD_NUMBER: builtins.int
    USE_NON_LAZY_ADAM_FIELD_NUMBER: builtins.int
    USE_SUM_INSIDE_SQRT_FIELD_NUMBER: builtins.int
    beta1: builtins.float
    beta2: builtins.float
    epsilon: builtins.float
    use_non_lazy_adam: builtins.bool
    use_sum_inside_sqrt: builtins.bool
    def __init__(
        self,
        *,
        beta1: builtins.float | None = ...,
        beta2: builtins.float | None = ...,
        epsilon: builtins.float | None = ...,
        use_non_lazy_adam: builtins.bool | None = ...,
        use_sum_inside_sqrt: builtins.bool | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "beta1",
            b"beta1",
            "beta2",
            b"beta2",
            "epsilon",
            b"epsilon",
            "use_non_lazy_adam",
            b"use_non_lazy_adam",
            "use_sum_inside_sqrt",
            b"use_sum_inside_sqrt",
        ],
    ) -> None: ...

global___AdamParameters = AdamParameters

@typing.final
class MomentumParameters(google.protobuf.message.Message):
    """https://www.tensorflow.org/api_docs/python/tf/keras/optimizers/SGD
    https://github.com/tensorflow/tensorflow/blob/6b6471f3ffb7f1fefe42d814aa5fb9ab7a535b58/tensorflow/core/kernels/training_ops.cc#L3068
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MOMENTUM_FIELD_NUMBER: builtins.int
    USE_NESTEROV_FIELD_NUMBER: builtins.int
    momentum: builtins.float
    use_nesterov: builtins.bool
    def __init__(self, *, momentum: builtins.float | None = ..., use_nesterov: builtins.bool | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["momentum", b"momentum", "use_nesterov", b"use_nesterov"]) -> None: ...

global___MomentumParameters = MomentumParameters

@typing.final
class LionParameters(google.protobuf.message.Message):
    """https://www.tensorflow.org/api_docs/python/tf/keras/optimizers/Lion
    momenta(new) = beta2 * momenta(old) + (1 - beta2) * grad
    momenta_t = beta1 * momenta(old) + (1 - beta1) * grad
    var(new) = var(old) - lr * sign(momenta_t)
    Algorithm described in https://arxiv.org/abs/2302.06675.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BETA1_FIELD_NUMBER: builtins.int
    BETA2_FIELD_NUMBER: builtins.int
    USE_NON_LAZY_LION_FIELD_NUMBER: builtins.int
    beta1: builtins.float
    beta2: builtins.float
    use_non_lazy_lion: builtins.bool
    def __init__(
        self,
        *,
        beta1: builtins.float | None = ...,
        beta2: builtins.float | None = ...,
        use_non_lazy_lion: builtins.bool | None = ...,
    ) -> None: ...
    def ClearField(
        self, field_name: typing.Literal["beta1", b"beta1", "beta2", b"beta2", "use_non_lazy_lion", b"use_non_lazy_lion"]
    ) -> None: ...

global___LionParameters = LionParameters

@typing.final
class RmsPropParameters(google.protobuf.message.Message):
    """https://www.tensorflow.org/api_docs/python/tf/keras/optimizers/RMSprop
    https://github.com/tensorflow/tensorflow/blob/6b6471f3ffb7f1fefe42d814aa5fb9ab7a535b58/tensorflow/core/kernels/training_ops.cc#L4229
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RHO_FIELD_NUMBER: builtins.int
    MOMENTUM_FIELD_NUMBER: builtins.int
    EPSILON_FIELD_NUMBER: builtins.int
    rho: builtins.float
    momentum: builtins.float
    epsilon: builtins.float
    def __init__(
        self, *, rho: builtins.float | None = ..., momentum: builtins.float | None = ..., epsilon: builtins.float | None = ...
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["epsilon", b"epsilon", "momentum", b"momentum", "rho", b"rho"]) -> None: ...

global___RmsPropParameters = RmsPropParameters

@typing.final
class CenteredRmsPropParameters(google.protobuf.message.Message):
    """https://www.tensorflow.org/api_docs/python/tf/keras/optimizers/RMSprop
    https://github.com/tensorflow/tensorflow/blob/6b6471f3ffb7f1fefe42d814aa5fb9ab7a535b58/tensorflow/core/kernels/training_ops.cc#L4358
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RHO_FIELD_NUMBER: builtins.int
    MOMENTUM_FIELD_NUMBER: builtins.int
    EPSILON_FIELD_NUMBER: builtins.int
    rho: builtins.float
    momentum: builtins.float
    epsilon: builtins.float
    def __init__(
        self, *, rho: builtins.float | None = ..., momentum: builtins.float | None = ..., epsilon: builtins.float | None = ...
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["epsilon", b"epsilon", "momentum", b"momentum", "rho", b"rho"]) -> None: ...

global___CenteredRmsPropParameters = CenteredRmsPropParameters

@typing.final
class MdlAdagradLightParameters(google.protobuf.message.Message):
    """Variant of algorithm in http://proceedings.mlr.press/v44/shamir15.pdf"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    L2_FIELD_NUMBER: builtins.int
    LR_POWER_FIELD_NUMBER: builtins.int
    MIN_SERVABLE_MDL_BENEFIT_FIELD_NUMBER: builtins.int
    MDL_MIX_IN_MARGIN_FIELD_NUMBER: builtins.int
    MDL_BENEFIT_RAMPUP_COEFF_FIELD_NUMBER: builtins.int
    MDL_MIN_WEIGHT_FIELD_NUMBER: builtins.int
    BENEFIT_REVISIT_SCALE_FIELD_NUMBER: builtins.int
    MAX_EVENT_BENEFIT_FIELD_NUMBER: builtins.int
    MAX_TOTAL_BENEFIT_FIELD_NUMBER: builtins.int
    MDL_HARD_LIMIT_FIELD_NUMBER: builtins.int
    HARD_LIMIT_MIN_BENEFIT_FIELD_NUMBER: builtins.int
    MDL_REGULARIZE_FIELD_NUMBER: builtins.int
    l2: builtins.float
    lr_power: builtins.float
    min_servable_mdl_benefit: builtins.float
    mdl_mix_in_margin: builtins.float
    mdl_benefit_rampup_coeff: builtins.float
    mdl_min_weight: builtins.float
    benefit_revisit_scale: builtins.float
    max_event_benefit: builtins.float
    max_total_benefit: builtins.float
    mdl_hard_limit: builtins.float
    hard_limit_min_benefit: builtins.bool
    mdl_regularize: builtins.bool
    def __init__(
        self,
        *,
        l2: builtins.float | None = ...,
        lr_power: builtins.float | None = ...,
        min_servable_mdl_benefit: builtins.float | None = ...,
        mdl_mix_in_margin: builtins.float | None = ...,
        mdl_benefit_rampup_coeff: builtins.float | None = ...,
        mdl_min_weight: builtins.float | None = ...,
        benefit_revisit_scale: builtins.float | None = ...,
        max_event_benefit: builtins.float | None = ...,
        max_total_benefit: builtins.float | None = ...,
        mdl_hard_limit: builtins.float | None = ...,
        hard_limit_min_benefit: builtins.bool | None = ...,
        mdl_regularize: builtins.bool | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "benefit_revisit_scale",
            b"benefit_revisit_scale",
            "hard_limit_min_benefit",
            b"hard_limit_min_benefit",
            "l2",
            b"l2",
            "lr_power",
            b"lr_power",
            "max_event_benefit",
            b"max_event_benefit",
            "max_total_benefit",
            b"max_total_benefit",
            "mdl_benefit_rampup_coeff",
            b"mdl_benefit_rampup_coeff",
            "mdl_hard_limit",
            b"mdl_hard_limit",
            "mdl_min_weight",
            b"mdl_min_weight",
            "mdl_mix_in_margin",
            b"mdl_mix_in_margin",
            "mdl_regularize",
            b"mdl_regularize",
            "min_servable_mdl_benefit",
            b"min_servable_mdl_benefit",
        ],
    ) -> None: ...

global___MdlAdagradLightParameters = MdlAdagradLightParameters

@typing.final
class AdadeltaParameters(google.protobuf.message.Message):
    """https://www.tensorflow.org/api_docs/python/tf/keras/optimizers/Adadelta
    https://github.com/tensorflow/tensorflow/blob/6b6471f3ffb7f1fefe42d814aa5fb9ab7a535b58/tensorflow/core/kernels/training_ops.cc#L933
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RHO_FIELD_NUMBER: builtins.int
    EPSILON_FIELD_NUMBER: builtins.int
    rho: builtins.float
    epsilon: builtins.float
    def __init__(self, *, rho: builtins.float | None = ..., epsilon: builtins.float | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["epsilon", b"epsilon", "rho", b"rho"]) -> None: ...

global___AdadeltaParameters = AdadeltaParameters

@typing.final
class ProximalAdagradParameters(google.protobuf.message.Message):
    """https://www.tensorflow.org/api_docs/python/tf/compat/v1/train/ProximalAdagradOptimizer
    https://github.com/tensorflow/tensorflow/blob/6b6471f3ffb7f1fefe42d814aa5fb9ab7a535b58/tensorflow/core/kernels/training_ops.cc#L1961
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    L1_FIELD_NUMBER: builtins.int
    L2_FIELD_NUMBER: builtins.int
    l1: builtins.float
    l2: builtins.float
    def __init__(self, *, l1: builtins.float | None = ..., l2: builtins.float | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["l1", b"l1", "l2", b"l2"]) -> None: ...

global___ProximalAdagradParameters = ProximalAdagradParameters

@typing.final
class OnlineYogiParameters(google.protobuf.message.Message):
    """The online Yogi optimizer does not implement hyper-parameter update; use the
    dynamic learning rate feature instead, setting the learning rate to:
    user learning_rate * sqrt(1 - beta2^t) / (1 - beta1^t)
    Here, t is the current timestep.

    https://papers.nips.cc/paper/8186-adaptive-methods-for-nonconvex-optimization.pdf
    plus some extensions based on FTRL.

    Note that the code by default implements the lazy version of online Yogi.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    L1_FIELD_NUMBER: builtins.int
    L2_FIELD_NUMBER: builtins.int
    BETA2_FIELD_NUMBER: builtins.int
    l1: builtins.float
    """The L1 regularization parameter (used analogously to the one in FTRL)."""
    l2: builtins.float
    """The L2 regularization parameter (used analogously to the one in FTRL)."""
    beta2: builtins.float
    """\\beta_2 from Algorithm 2 in the paper."""
    def __init__(
        self, *, l1: builtins.float | None = ..., l2: builtins.float | None = ..., beta2: builtins.float | None = ...
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["beta2", b"beta2", "l1", b"l1", "l2", b"l2"]) -> None: ...

global___OnlineYogiParameters = OnlineYogiParameters

@typing.final
class ProximalYogiParameters(google.protobuf.message.Message):
    """The online Yogi optimizer does not implement hyper-parameter update; use the
    dynamic learning rate feature instead, setting the learning rate to:
    user learning_rate * sqrt(1 - beta2^t) / (1 - beta1^t)
    Here, t is the current timestep.

    https://papers.nips.cc/paper/8186-adaptive-methods-for-nonconvex-optimization.pdf
    plus some extensions based on FTRL.

    Note that the code by default implements the lazy version of proximal Yogi.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    L1_FIELD_NUMBER: builtins.int
    L2_FIELD_NUMBER: builtins.int
    BETA1_FIELD_NUMBER: builtins.int
    BETA2_FIELD_NUMBER: builtins.int
    EPSILON_FIELD_NUMBER: builtins.int
    l1: builtins.float
    """The L1 regularization parameter."""
    l2: builtins.float
    """The L2 regularization parameter."""
    beta1: builtins.float
    """The exponential decay rate for the 1st moment estimates."""
    beta2: builtins.float
    """The exponential decay rate for the 2nd moment estimates."""
    epsilon: builtins.float
    """A constant trading off adaptivity and noise."""
    def __init__(
        self,
        *,
        l1: builtins.float | None = ...,
        l2: builtins.float | None = ...,
        beta1: builtins.float | None = ...,
        beta2: builtins.float | None = ...,
        epsilon: builtins.float | None = ...,
    ) -> None: ...
    def ClearField(
        self, field_name: typing.Literal["beta1", b"beta1", "beta2", b"beta2", "epsilon", b"epsilon", "l1", b"l1", "l2", b"l2"]
    ) -> None: ...

global___ProximalYogiParameters = ProximalYogiParameters

@typing.final
class FrequencyEstimatorParameters(google.protobuf.message.Message):
    """Estimator for the frequency of updates to a lookup table. It maintains an
    array (tf.Variable) D, where each element records the average number of
    global steps between two consecutive batches that hit the corresponding
    bucket. Once an item with bucket id i is sampled, D[i] is updated by:
      D[i] <- D[i] * (1 - tau) + delta[i] * tau,

    where tau is a learning rate between 0 and 1 (exclusive), and
      delta[i] = current global step - last step i is sampled.

    The estimated frequency (sampling rate in a batch) is thus 1 / D[i].

    Elements in D are initialized with a large value max_delta. delta[i] will
    also be capped by this value.

    The exact sequence of operations used in the optimizer is shown below.
    last_hit_step[i] is a tf.Variable that holds the last global step at which i
    was sampled.

      delta = global_step - last_hit_step[i]
      clipped_delta = min(delta, params.max_delta)
      is_outlier = (delta >= params.outlier_threshold * D[i])
      D[i] <- is_outlier ? clipped_delta
                         : D[i] * (1 - params.tau) + clipped_delta * params.tau
      last_hit_step[i] <- global_step
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TAU_FIELD_NUMBER: builtins.int
    MAX_DELTA_FIELD_NUMBER: builtins.int
    OUTLIER_THRESHOLD_FIELD_NUMBER: builtins.int
    WEIGHT_EXPONENT_FIELD_NUMBER: builtins.int
    tau: builtins.float
    """Learning rate between (0, 1) that is used to update the array D."""
    max_delta: builtins.float
    """Maximum value of delta: difference between the current global step and the
    last global step at which the row was sampled.
    """
    outlier_threshold: builtins.float
    """Threshold used to determine whether the current update is an outlier."""
    weight_exponent: builtins.float
    """The weight exponent used to transform the estimated delta into weights.
    The transformation function is: (delta / max_delta) ^ (weight_exponent)
    """
    def __init__(
        self,
        *,
        tau: builtins.float | None = ...,
        max_delta: builtins.float | None = ...,
        outlier_threshold: builtins.float | None = ...,
        weight_exponent: builtins.float | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "max_delta",
            b"max_delta",
            "outlier_threshold",
            b"outlier_threshold",
            "tau",
            b"tau",
            "weight_exponent",
            b"weight_exponent",
        ],
    ) -> None: ...

global___FrequencyEstimatorParameters = FrequencyEstimatorParameters

@typing.final
class UserDefinedProgramParameters(google.protobuf.message.Message):
    """A user-defined optimizer.
    The contained HLO program must take the following arguments in the following
    order:
    1.  gradients
    2.  table weights
    3.  slot variables
    4.  an optional scalar input that is passed in via the dynamic learning
        rate mechanism.

    It must return/end in a tuple op that contains the following values in the
    following order:
    1.  new table values
    2.  new slot variable value

    The program must have shape (1,1) with dtype float32 throughout and only use
    HLO that operate elementwise (e.g., no reduce, no variables, no control flow
    and no broadcasting outside of the single scalar input).
    The HLO program should be written as if it were a dense update. It will be
    called on each row that needs an update and will applied elementwise.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROGRAM_FIELD_NUMBER: builtins.int
    @property
    def program(self) -> tensorflow.compiler.xla.service.hlo_pb2.HloModuleProto: ...
    def __init__(self, *, program: tensorflow.compiler.xla.service.hlo_pb2.HloModuleProto | None = ...) -> None: ...
    def HasField(self, field_name: typing.Literal["program", b"program"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["program", b"program"]) -> None: ...

global___UserDefinedProgramParameters = UserDefinedProgramParameters

@typing.final
class AssignParameters(google.protobuf.message.Message):
    """Optimizer that just sets the variable to the value of the gradient. To be
    correct, this requires either gradient accumulation (to sum the values of a
    computed expression across the samples) or to deduplicate IDs within a single
    host (to assign the value from an arbitrary sample).
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(self) -> None: ...

global___AssignParameters = AssignParameters

@typing.final
class GradientAccumulationStatus(google.protobuf.message.Message):
    """Status of using gradient accumulation (doing two passes over the input
    gradients: one to accumulate them into a temporary array and another to apply
    them using the actual optimization algorithm). The extra message is to wrap
    the enum for scoping.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Status:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _StatusEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[GradientAccumulationStatus._Status.ValueType], builtins.type
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UNSPECIFIED: GradientAccumulationStatus._Status.ValueType  # 0
        ENABLED: GradientAccumulationStatus._Status.ValueType  # 1
        DISABLED: GradientAccumulationStatus._Status.ValueType  # 2

    class Status(_Status, metaclass=_StatusEnumTypeWrapper):
        """if UNSPECIFIED (default), gradient accumulation is ENABLED."""

    UNSPECIFIED: GradientAccumulationStatus.Status.ValueType  # 0
    ENABLED: GradientAccumulationStatus.Status.ValueType  # 1
    DISABLED: GradientAccumulationStatus.Status.ValueType  # 2

    def __init__(self) -> None: ...

global___GradientAccumulationStatus = GradientAccumulationStatus

@typing.final
class LowDimensionalPackingStatus(google.protobuf.message.Message):
    """Whether to optimize the packing of low-dimensional embedding tables in HBM
    (high bandwidth memory). TPUs access HBM at 32-byte (8-float) granularity.
    For functional correctness, the TPU software internally pads the embedding
    dimension to a multiple of 8. This can sometimes lead to significant memory
    wastage due to padding. For 1-dimensional, 2-dimensional, and 4-dimensional,
    the TPU software can remove this padding by packing multiple rows into the
    same 8-float HBM chunk. For example, 8 rows could be packed into the same
    8-float chunk for a 1-dimensional embedding table.

    There is one important limitation for this HBM packing though. When only a
    subset of rows in an 8-float chunk are accessed on a particular step, the
    adjoining rows in the same chunk are updated with zero gradients on the
    backward pass even if they are not touched. This is an artifact of the
    packing implementation. This operation is NOT functionally correct for
    optimizers where zero gradients change the embeddings/slot-variable values,
    e.g., momentum-based optimizers. Hence, this HBM packing cannot be enabled
    for embedding tables with such optimizers. The TPU software automatically
    recognizes that a zero gradient can modify state and turns off the low
    dimensional embedding packing in that scenario.

    However, for optimizers where a zero gradient is a NoOp, such as SGD,
    Adagrad, and FTRL, this packing optimization can be used. However, there are
    some important considerations:
    * Clipping limits: The initial values for such embeddings should fall within
      the clipping limits specified in the optimization parameters. Otherwise, a
      zero gradient will cause the embeddings to be clipped. This changes state
      and hence, is not a NoOp.
    * FTRL: The embedding vector is computed directly from the values of the
      accumulator and linear slot variables. Hence, the initial embedding values
      should match that computed from the initial values of the accumulator and
      linear slot variables. Note that in nearly all cases, the linear value is
      initialized to zero; this corresponds to an embedding value of zero.

    Performance: The TPU has to perform additional work when low dimensional
    packing is enabled. In certain situations when the vocabulary size is small,
    it may not make sense to turn on this packing since the total memory usage
    due to padding is extremely low. Hence, the TPU software automatically turns
    off the packing optimization in such scenarios.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Status:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _StatusEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[LowDimensionalPackingStatus._Status.ValueType], builtins.type
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UNSPECIFIED: LowDimensionalPackingStatus._Status.ValueType  # 0
        ENABLED: LowDimensionalPackingStatus._Status.ValueType  # 1
        DISABLED: LowDimensionalPackingStatus._Status.ValueType  # 2

    class Status(_Status, metaclass=_StatusEnumTypeWrapper):
        """if UNSPECIFIED (default), the low dimension packing status is DISABLED.
        This can change in future.

        if ENABLED, the low dimension packing is enabled only if the following
        three additional conditions are true:
         * The optimizer treats the zero gradient as a NoOp.
         * The embedding dimension is 1, 2, or 4.
         * The vocabulary size is large enough to avoid performance issues.

        if DISABLED, the low dimension packing is always disabled.
        """

    UNSPECIFIED: LowDimensionalPackingStatus.Status.ValueType  # 0
    ENABLED: LowDimensionalPackingStatus.Status.ValueType  # 1
    DISABLED: LowDimensionalPackingStatus.Status.ValueType  # 2

    def __init__(self) -> None: ...

global___LowDimensionalPackingStatus = LowDimensionalPackingStatus

@typing.final
class HotIdReplicationConfiguration(google.protobuf.message.Message):
    """Configuration proto for hot ID optimization. This is an experimental feature
    that is currently disabled (by default).
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Status:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _StatusEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[HotIdReplicationConfiguration._Status.ValueType],
        builtins.type,
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UNSPECIFIED: HotIdReplicationConfiguration._Status.ValueType  # 0
        ENABLED: HotIdReplicationConfiguration._Status.ValueType  # 1
        DISABLED: HotIdReplicationConfiguration._Status.ValueType  # 2
        MIGRATION_ONLY: HotIdReplicationConfiguration._Status.ValueType  # 3

    class Status(_Status, metaclass=_StatusEnumTypeWrapper):
        """Whether to enable or disable hot ID optimization.
        If set to UNSPECIFIED (default), hot ID optimization is DISABLED.
        If set to ENABLED, hot ID replication is turned ON.
        If set to MIGRATION_ONLY, hot ID migration is turned ON.
        """

    UNSPECIFIED: HotIdReplicationConfiguration.Status.ValueType  # 0
    ENABLED: HotIdReplicationConfiguration.Status.ValueType  # 1
    DISABLED: HotIdReplicationConfiguration.Status.ValueType  # 2
    MIGRATION_ONLY: HotIdReplicationConfiguration.Status.ValueType  # 3

    STATUS_FIELD_NUMBER: builtins.int
    status: global___HotIdReplicationConfiguration.Status.ValueType
    def __init__(self, *, status: global___HotIdReplicationConfiguration.Status.ValueType | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["status", b"status"]) -> None: ...

global___HotIdReplicationConfiguration = HotIdReplicationConfiguration

@typing.final
class OptimizationParameters(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LEARNING_RATE_FIELD_NUMBER: builtins.int
    CLIPPING_LIMITS_FIELD_NUMBER: builtins.int
    GRADIENT_CLIPPING_LIMITS_FIELD_NUMBER: builtins.int
    WEIGHT_DECAY_FACTOR_FIELD_NUMBER: builtins.int
    MULTIPLY_WEIGHT_DECAY_FACTOR_BY_LEARNING_RATE_FIELD_NUMBER: builtins.int
    SIMULATED_QUANTIZATION_FIELD_NUMBER: builtins.int
    GRADIENT_ACCUMULATION_STATUS_FIELD_NUMBER: builtins.int
    LOW_DIMENSIONAL_PACKING_STATUS_FIELD_NUMBER: builtins.int
    HOT_ID_REPLICATION_CONFIGURATION_FIELD_NUMBER: builtins.int
    ADAGRAD_FIELD_NUMBER: builtins.int
    ADAGRAD_MOMENTUM_FIELD_NUMBER: builtins.int
    BOUNDED_ADAGRAD_FIELD_NUMBER: builtins.int
    STOCHASTIC_GRADIENT_DESCENT_FIELD_NUMBER: builtins.int
    FTRL_FIELD_NUMBER: builtins.int
    ADAM_FIELD_NUMBER: builtins.int
    MOMENTUM_FIELD_NUMBER: builtins.int
    LION_FIELD_NUMBER: builtins.int
    RMS_PROP_FIELD_NUMBER: builtins.int
    CENTERED_RMS_PROP_FIELD_NUMBER: builtins.int
    MDL_ADAGRAD_LIGHT_FIELD_NUMBER: builtins.int
    ADADELTA_FIELD_NUMBER: builtins.int
    PROXIMAL_ADAGRAD_FIELD_NUMBER: builtins.int
    ONLINE_YOGI_FIELD_NUMBER: builtins.int
    PROXIMAL_YOGI_FIELD_NUMBER: builtins.int
    FREQUENCY_ESTIMATOR_FIELD_NUMBER: builtins.int
    USER_DEFINED_PROGRAM_FIELD_NUMBER: builtins.int
    ASSIGN_FIELD_NUMBER: builtins.int
    weight_decay_factor: builtins.float
    """Amount of weight decay to apply; see weight_decay_optimizers.py for
    details. All optimizers except MDL Adagrad Light are supported with this
    option. Although there is no check, users who want weight decay will also
    want to ensure that gradient accumulation is enabled so that the decay will
    happen once per global batch.
    """
    multiply_weight_decay_factor_by_learning_rate: builtins.bool
    """If true, the weight decay factor is multiplied by the current learning rate
    before use; this is to match the note in DecoupledWeightDecayExtension in
    weight_decay_optimizers.py.
    """
    gradient_accumulation_status: global___GradientAccumulationStatus.Status.ValueType
    """Status of using gradient accumulation (doing two passes over the input
    gradients: one to accumulate them into a temporary array and another to
    apply them using the actual optimization algorithm).
    """
    low_dimensional_packing_status: global___LowDimensionalPackingStatus.Status.ValueType
    """Status of the low-dimensional embedding packing optimization. This controls
    whether to optimize the packing of 1-dimensional, 2-dimensional, and
    4-dimensional embedding tables in memory.
    """
    @property
    def learning_rate(self) -> global___LearningRate:
        """Learning rate used for updating the embedding layer parameters."""

    @property
    def clipping_limits(self) -> global___ClippingLimits:
        """Limits to which to clip the weight values after the backward pass; not
        present means no limits are applied.
        """

    @property
    def gradient_clipping_limits(self) -> global___ClippingLimits:
        """Limits to which to clip the backward pass gradient before using it for
        updates; not present means no limits are applied.
        """

    @property
    def simulated_quantization(self) -> global___SimulatedQuantization:
        """Configuration for simulated quantization which is used to reduce
        training/serving skew when the serving variables are quantized. The same
        quantization operations are executed during training to minimize
        differences with serving.
        """

    @property
    def hot_id_replication_configuration(self) -> global___HotIdReplicationConfiguration:
        """Configuration proto for hot ID replication. This is an experimental
        feature that is currently disabled (by default).
        """

    @property
    def adagrad(self) -> global___AdagradParameters: ...
    @property
    def adagrad_momentum(self) -> global___AdagradMomentumParameters: ...
    @property
    def bounded_adagrad(self) -> global___BoundedAdagradParameters: ...
    @property
    def stochastic_gradient_descent(self) -> global___StochasticGradientDescentParameters: ...
    @property
    def ftrl(self) -> global___FtrlParameters: ...
    @property
    def adam(self) -> global___AdamParameters: ...
    @property
    def momentum(self) -> global___MomentumParameters: ...
    @property
    def lion(self) -> global___LionParameters: ...
    @property
    def rms_prop(self) -> global___RmsPropParameters: ...
    @property
    def centered_rms_prop(self) -> global___CenteredRmsPropParameters: ...
    @property
    def mdl_adagrad_light(self) -> global___MdlAdagradLightParameters: ...
    @property
    def adadelta(self) -> global___AdadeltaParameters: ...
    @property
    def proximal_adagrad(self) -> global___ProximalAdagradParameters: ...
    @property
    def online_yogi(self) -> global___OnlineYogiParameters: ...
    @property
    def proximal_yogi(self) -> global___ProximalYogiParameters: ...
    @property
    def frequency_estimator(self) -> global___FrequencyEstimatorParameters: ...
    @property
    def user_defined_program(self) -> global___UserDefinedProgramParameters: ...
    @property
    def assign(self) -> global___AssignParameters: ...
    def __init__(
        self,
        *,
        learning_rate: global___LearningRate | None = ...,
        clipping_limits: global___ClippingLimits | None = ...,
        gradient_clipping_limits: global___ClippingLimits | None = ...,
        weight_decay_factor: builtins.float | None = ...,
        multiply_weight_decay_factor_by_learning_rate: builtins.bool | None = ...,
        simulated_quantization: global___SimulatedQuantization | None = ...,
        gradient_accumulation_status: global___GradientAccumulationStatus.Status.ValueType | None = ...,
        low_dimensional_packing_status: global___LowDimensionalPackingStatus.Status.ValueType | None = ...,
        hot_id_replication_configuration: global___HotIdReplicationConfiguration | None = ...,
        adagrad: global___AdagradParameters | None = ...,
        adagrad_momentum: global___AdagradMomentumParameters | None = ...,
        bounded_adagrad: global___BoundedAdagradParameters | None = ...,
        stochastic_gradient_descent: global___StochasticGradientDescentParameters | None = ...,
        ftrl: global___FtrlParameters | None = ...,
        adam: global___AdamParameters | None = ...,
        momentum: global___MomentumParameters | None = ...,
        lion: global___LionParameters | None = ...,
        rms_prop: global___RmsPropParameters | None = ...,
        centered_rms_prop: global___CenteredRmsPropParameters | None = ...,
        mdl_adagrad_light: global___MdlAdagradLightParameters | None = ...,
        adadelta: global___AdadeltaParameters | None = ...,
        proximal_adagrad: global___ProximalAdagradParameters | None = ...,
        online_yogi: global___OnlineYogiParameters | None = ...,
        proximal_yogi: global___ProximalYogiParameters | None = ...,
        frequency_estimator: global___FrequencyEstimatorParameters | None = ...,
        user_defined_program: global___UserDefinedProgramParameters | None = ...,
        assign: global___AssignParameters | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "adadelta",
            b"adadelta",
            "adagrad",
            b"adagrad",
            "adagrad_momentum",
            b"adagrad_momentum",
            "adam",
            b"adam",
            "assign",
            b"assign",
            "bounded_adagrad",
            b"bounded_adagrad",
            "centered_rms_prop",
            b"centered_rms_prop",
            "clipping_limits",
            b"clipping_limits",
            "frequency_estimator",
            b"frequency_estimator",
            "ftrl",
            b"ftrl",
            "gradient_clipping_limits",
            b"gradient_clipping_limits",
            "hot_id_replication_configuration",
            b"hot_id_replication_configuration",
            "learning_rate",
            b"learning_rate",
            "lion",
            b"lion",
            "mdl_adagrad_light",
            b"mdl_adagrad_light",
            "momentum",
            b"momentum",
            "online_yogi",
            b"online_yogi",
            "parameters",
            b"parameters",
            "proximal_adagrad",
            b"proximal_adagrad",
            "proximal_yogi",
            b"proximal_yogi",
            "rms_prop",
            b"rms_prop",
            "simulated_quantization",
            b"simulated_quantization",
            "stochastic_gradient_descent",
            b"stochastic_gradient_descent",
            "user_defined_program",
            b"user_defined_program",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "adadelta",
            b"adadelta",
            "adagrad",
            b"adagrad",
            "adagrad_momentum",
            b"adagrad_momentum",
            "adam",
            b"adam",
            "assign",
            b"assign",
            "bounded_adagrad",
            b"bounded_adagrad",
            "centered_rms_prop",
            b"centered_rms_prop",
            "clipping_limits",
            b"clipping_limits",
            "frequency_estimator",
            b"frequency_estimator",
            "ftrl",
            b"ftrl",
            "gradient_accumulation_status",
            b"gradient_accumulation_status",
            "gradient_clipping_limits",
            b"gradient_clipping_limits",
            "hot_id_replication_configuration",
            b"hot_id_replication_configuration",
            "learning_rate",
            b"learning_rate",
            "lion",
            b"lion",
            "low_dimensional_packing_status",
            b"low_dimensional_packing_status",
            "mdl_adagrad_light",
            b"mdl_adagrad_light",
            "momentum",
            b"momentum",
            "multiply_weight_decay_factor_by_learning_rate",
            b"multiply_weight_decay_factor_by_learning_rate",
            "online_yogi",
            b"online_yogi",
            "parameters",
            b"parameters",
            "proximal_adagrad",
            b"proximal_adagrad",
            "proximal_yogi",
            b"proximal_yogi",
            "rms_prop",
            b"rms_prop",
            "simulated_quantization",
            b"simulated_quantization",
            "stochastic_gradient_descent",
            b"stochastic_gradient_descent",
            "user_defined_program",
            b"user_defined_program",
            "weight_decay_factor",
            b"weight_decay_factor",
        ],
    ) -> None: ...
    def WhichOneof(
        self, oneof_group: typing.Literal["parameters", b"parameters"]
    ) -> (
        typing.Literal[
            "adagrad",
            "adagrad_momentum",
            "bounded_adagrad",
            "stochastic_gradient_descent",
            "ftrl",
            "adam",
            "momentum",
            "lion",
            "rms_prop",
            "centered_rms_prop",
            "mdl_adagrad_light",
            "adadelta",
            "proximal_adagrad",
            "online_yogi",
            "proximal_yogi",
            "frequency_estimator",
            "user_defined_program",
            "assign",
        ]
        | None
    ): ...

global___OptimizationParameters = OptimizationParameters

@typing.final
class StateVariableSpecification(google.protobuf.message.Message):
    """Specification of an optimization algorithm's state variables (both the main
    value vector and any extra accumulators, etc.). This proto is only used
    internally by the TPU software and is not exposed directly to the TF model.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class UserDefined(google.protobuf.message.Message):
        """A normal state variable that should be saved and restored in checkpoints
        and used as an input or output to non-debug TensorFlow ops.
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        def __init__(self) -> None: ...

    @typing.final
    class FillWithConstant(google.protobuf.message.Message):
        """A state variable that should be filled with a constant and normally hidden
        from users (used for intermediate gradients being accumulated, for
        example).
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        INITIAL_VALUE_FIELD_NUMBER: builtins.int
        initial_value: builtins.float
        def __init__(self, *, initial_value: builtins.float | None = ...) -> None: ...
        def ClearField(self, field_name: typing.Literal["initial_value", b"initial_value"]) -> None: ...

    NAME_FIELD_NUMBER: builtins.int
    USER_DEFINED_FIELD_NUMBER: builtins.int
    FILL_WITH_CONSTANT_FIELD_NUMBER: builtins.int
    name: builtins.str
    """Parameter name for the state variable."""
    @property
    def user_defined(self) -> global___StateVariableSpecification.UserDefined: ...
    @property
    def fill_with_constant(self) -> global___StateVariableSpecification.FillWithConstant: ...
    def __init__(
        self,
        *,
        name: builtins.str | None = ...,
        user_defined: global___StateVariableSpecification.UserDefined | None = ...,
        fill_with_constant: global___StateVariableSpecification.FillWithConstant | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "fill_with_constant", b"fill_with_constant", "usage", b"usage", "user_defined", b"user_defined"
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "fill_with_constant", b"fill_with_constant", "name", b"name", "usage", b"usage", "user_defined", b"user_defined"
        ],
    ) -> None: ...
    def WhichOneof(
        self, oneof_group: typing.Literal["usage", b"usage"]
    ) -> typing.Literal["user_defined", "fill_with_constant"] | None: ...

global___StateVariableSpecification = StateVariableSpecification



from gevent._types import _AddrinfoResult, _NameinfoResult, _SockAddr
from gevent.hub import Hub

class Resolver:
    def __init__(self, hub: Hub | None = None) -> None: ...
    def close(self) -> None: ...
    def gethostbyname(self, hostname: str, family: int = 2) -> str: ...
    def gethostbyname_ex(self, hostname: str, family: int = 2) -> tuple[str, list[str], list[str]]: ...
    def getaddrinfo(
        self, host: str, port: int, family: int = 0, socktype: int = 0, proto: int = 0, flags: int = 0
    ) -> _AddrinfoResult: ...
    def gethostbyaddr(self, ip_address: str) -> tuple[str, list[str], list[str]]: ...
    def getnameinfo(self, sockaddr: _SockAddr, flags: int) -> _NameinfoResult: ...

__all__ = ["Resolver"]



from _typeshed import Incomplete
from typing import Final

from reportlab.graphics.widgets.signsandsymbols import _Symbol
from reportlab.lib.attrmap import *
from reportlab.lib.validators import *

__version__: Final[str]
validFlag: Incomplete

class Star(_Symbol):
    size: int
    fillColor: Incomplete
    strokeColor: Incomplete
    angle: int
    def __init__(self) -> None: ...
    def demo(self): ...
    def draw(self): ...

class Flag(_Symbol):
    kind: Incomplete
    size: int
    fillColor: Incomplete
    border: int
    def __init__(self, **kw) -> None: ...
    def availableFlagNames(self): ...
    def draw(self): ...
    def clone(self): ...
    def demo(self): ...

def makeFlag(name): ...
def test() -> None: ...



from collections.abc import Collection

class ClientMixin:
    def get_client_id(self) -> str: ...
    def get_default_redirect_uri(self) -> str: ...
    def get_allowed_scope(self, scope: Collection[str] | str) -> str: ...
    def check_redirect_uri(self, redirect_uri: str) -> bool: ...
    def check_client_secret(self, client_secret: str) -> bool: ...
    def check_endpoint_auth_method(self, method: str, endpoint: str) -> bool: ...
    def check_response_type(self, response_type: str) -> bool: ...
    def check_grant_type(self, grant_type: str) -> bool: ...

class AuthorizationCodeMixin:
    def get_redirect_uri(self) -> str: ...
    def get_scope(self) -> str: ...

class TokenMixin:
    def check_client(self, client) -> bool: ...
    def get_scope(self) -> str: ...
    def get_expires_in(self) -> int: ...
    def is_expired(self) -> bool: ...
    def is_revoked(self) -> bool: ...
    def get_user(self): ...
    def get_client(self) -> ClientMixin: ...



from collections.abc import Container
from typing import Final

ALLOWED_CSS_PROPERTIES: Final[frozenset[str]]
ALLOWED_SVG_PROPERTIES: Final[frozenset[str]]

class CSSSanitizer:
    allowed_css_properties: Container[str]
    allowed_svg_properties: Container[str]

    def __init__(self, allowed_css_properties: Container[str] = ..., allowed_svg_properties: Container[str] = ...) -> None: ...
    def sanitize_css(self, style: str) -> str: ...



from typing import Any

class XRaySession: ...
class XRayQuery: ...

class XRaySessionMaker:
    def __init__(
        self,
        bind=None,
        class_: type = ...,
        autoflush: bool = True,
        autocommit: bool = False,
        expire_on_commit: bool = True,
        info: dict[Any, Any] | None = None,  # it was taken from sqlalchemy stubs
        **kw,
    ) -> None: ...



from _typeshed import Incomplete
from typing import Final

from reportlab.graphics.charts.areas import PlotArea
from reportlab.graphics.charts.piecharts import WedgeLabel
from reportlab.graphics.widgetbase import PropHolder
from reportlab.lib.attrmap import *

__version__: Final[str]

class StrandProperty(PropHolder):
    strokeWidth: int
    fillColor: Incomplete
    strokeColor: Incomplete
    strokeDashArray: Incomplete
    symbol: Incomplete
    symbolSize: int
    name: Incomplete
    def __init__(self) -> None: ...

class SpokeProperty(PropHolder):
    strokeWidth: float
    fillColor: Incomplete
    strokeColor: Incomplete
    strokeDashArray: Incomplete
    visible: int
    labelRadius: float
    def __init__(self, **kw) -> None: ...

class SpokeLabel(WedgeLabel):
    def __init__(self, **kw) -> None: ...

class StrandLabel(SpokeLabel):
    format: str
    dR: int
    def __init__(self, **kw) -> None: ...

class SpiderChart(PlotArea):
    def makeSwatchSample(self, rowNo, x, y, width, height): ...
    def getSeriesName(self, i, default=None): ...
    data: Incomplete
    labels: Incomplete
    startAngle: int
    direction: str
    strands: Incomplete
    spokes: Incomplete
    spokeLabels: Incomplete
    strandLabels: Incomplete
    x: int
    y: int
    width: int
    height: int
    def __init__(self) -> None: ...
    def demo(self): ...
    def normalizeData(self, outer: float = 0.0): ...
    def labelClass(self, kind): ...
    def draw(self): ...

def sample1(): ...
def sample2(): ...



from _typeshed import Incomplete, SupportsGetItem
from collections.abc import Callable
from typing import Any

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["voronoi_cells"]

@_dispatchable
def voronoi_cells(
    G: Graph[_Node],
    center_nodes: set[Incomplete],
    weight: str | Callable[[Any, Any, SupportsGetItem[str, Any]], float | None] | None = "weight",
) -> dict[Incomplete, set[Incomplete]]: ...



from _typeshed import Incomplete

Marker: Incomplete
listElementsMap: dict[str | None, tuple[frozenset[tuple[str, str]], bool]]

class Node:
    name: str
    parent: Incomplete
    value: Incomplete
    attributes: Incomplete
    childNodes: Incomplete
    def __init__(self, name: str) -> None: ...
    def appendChild(self, node) -> None: ...
    def insertText(self, data, insertBefore=None) -> None: ...
    def insertBefore(self, node, refNode) -> None: ...
    def removeChild(self, node) -> None: ...
    def reparentChildren(self, newParent) -> None: ...
    def cloneNode(self) -> None: ...
    def hasContent(self) -> None: ...

class ActiveFormattingElements(list[Incomplete]):
    def append(self, node) -> None: ...
    def nodesEqual(self, node1, node2) -> bool: ...

class TreeBuilder:
    documentClass: Incomplete
    elementClass: Incomplete
    commentClass: Incomplete
    doctypeClass: Incomplete
    fragmentClass: Incomplete
    defaultNamespace: str | None
    def __init__(self, namespaceHTMLElements: bool | None) -> None: ...
    openElements: Incomplete
    activeFormattingElements: Incomplete
    headPointer: Incomplete
    formPointer: Incomplete
    insertFromTable: bool
    document: Incomplete
    def reset(self) -> None: ...
    def elementInScope(self, target, variant=None): ...
    def reconstructActiveFormattingElements(self) -> None: ...
    def clearActiveFormattingElements(self) -> None: ...
    def elementInActiveFormattingElements(self, name): ...
    def insertRoot(self, token) -> None: ...
    def insertDoctype(self, token) -> None: ...
    def insertComment(self, token, parent=None) -> None: ...
    def createElement(self, token): ...
    def insertElementNormal(self, token): ...
    def insertElementTable(self, token): ...
    def insertText(self, data, parent=None) -> None: ...
    def getTableMisnestedNodePosition(self): ...
    def generateImpliedEndTags(self, exclude=None) -> None: ...
    def getDocument(self): ...
    def getFragment(self): ...
    def testSerializer(self, node): ...



from typing import Final

from .filters import *
from .filterset import FilterSet as FilterSet, UnknownFieldBehavior as UnknownFieldBehavior

__version__: Final[str]

def parse_version(version: str) -> tuple[str | int]: ...

VERSION: tuple[str | int, ...]



from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT: str

class Radius(VaultApiBase):
    def configure(
        self, host, secret, port=None, unregistered_user_policies=None, dial_timeout=None, nas_port=None, mount_point="radius"
    ): ...
    def read_configuration(self, mount_point="radius"): ...
    def register_user(self, username, policies=None, mount_point="radius"): ...
    def list_users(self, mount_point="radius"): ...
    def read_user(self, username, mount_point="radius"): ...
    def delete_user(self, username, mount_point="radius"): ...
    def login(self, username, password, use_token: bool = True, mount_point="radius"): ...



from _typeshed import Incomplete
from collections.abc import Iterable

from networkx.utils.backends import _dispatchable

__all__ = ["union_all", "compose_all", "disjoint_union_all", "intersection_all"]

@_dispatchable
def union_all(graphs: Iterable[Incomplete], rename: Iterable[Incomplete] | None = ()): ...
@_dispatchable
def disjoint_union_all(graphs: Iterable[Incomplete]): ...
@_dispatchable
def compose_all(graphs: Iterable[Incomplete]): ...
@_dispatchable
def intersection_all(graphs: Iterable[Incomplete]): ...



from . import version as version_mod
from .alarms import Alarms as Alarms, AlarmTime as AlarmTime
from .cal import (
    Alarm as Alarm,
    Calendar as Calendar,
    Component as Component,
    ComponentFactory as ComponentFactory,
    Event as Event,
    FreeBusy as FreeBusy,
    Journal as Journal,
    Timezone as Timezone,
    TimezoneDaylight as TimezoneDaylight,
    TimezoneStandard as TimezoneStandard,
    Todo as Todo,
)
from .enums import (
    CUTYPE as CUTYPE,
    FBTYPE as FBTYPE,
    PARTSTAT as PARTSTAT,
    RANGE as RANGE,
    RELATED as RELATED,
    RELTYPE as RELTYPE,
    ROLE as ROLE,
)
from .error import (
    ComponentEndMissing as ComponentEndMissing,
    ComponentStartMissing as ComponentStartMissing,
    FeatureWillBeRemovedInFutureVersion as FeatureWillBeRemovedInFutureVersion,
    IncompleteAlarmInformation as IncompleteAlarmInformation,
    IncompleteComponent as IncompleteComponent,
    InvalidCalendar as InvalidCalendar,
    LocalTimezoneMissing as LocalTimezoneMissing,
)
from .parser import Parameters as Parameters, q_join as q_join, q_split as q_split
from .prop import (
    TypesFactory as TypesFactory,
    vBinary as vBinary,
    vBoolean as vBoolean,
    vCalAddress as vCalAddress,
    vDate as vDate,
    vDatetime as vDatetime,
    vDDDLists as vDDDLists,
    vDDDTypes as vDDDTypes,
    vDuration as vDuration,
    vFloat as vFloat,
    vFrequency as vFrequency,
    vGeo as vGeo,
    vInt as vInt,
    vMonth as vMonth,
    vPeriod as vPeriod,
    vRecur as vRecur,
    vSkip as vSkip,
    vText as vText,
    vTime as vTime,
    vUri as vUri,
    vUTCOffset as vUTCOffset,
    vWeekday as vWeekday,
)
from .timezone import use_pytz, use_zoneinfo

__all__ = [
    "Calendar",
    "Event",
    "Todo",
    "Journal",
    "Timezone",
    "TimezoneStandard",
    "TimezoneDaylight",
    "FreeBusy",
    "Alarm",
    "ComponentFactory",
    "vBinary",
    "vBoolean",
    "vCalAddress",
    "vDatetime",
    "vDate",
    "vDDDLists",
    "vDDDTypes",
    "vDuration",
    "vFloat",
    "vInt",
    "vPeriod",
    "vWeekday",
    "vFrequency",
    "vRecur",
    "vText",
    "vTime",
    "vUri",
    "vGeo",
    "vUTCOffset",
    "Parameters",
    "q_split",
    "q_join",
    "use_pytz",
    "use_zoneinfo",
    "__version__",
    "version",
    "__version_tuple__",
    "version_tuple",
    "TypesFactory",
    "Component",
    "vMonth",
    "IncompleteComponent",
    "InvalidCalendar",
    "Alarms",
    "AlarmTime",
    "ComponentEndMissing",
    "ComponentStartMissing",
    "IncompleteAlarmInformation",
    "LocalTimezoneMissing",
    "CUTYPE",
    "FBTYPE",
    "PARTSTAT",
    "RANGE",
    "vSkip",
    "RELATED",
    "vSkip",
    "RELTYPE",
    "ROLE",
    "FeatureWillBeRemovedInFutureVersion",
]

__version__ = version_mod.__version__
__version_tuple__ = version_mod.__version_tuple__
version = version_mod.version
version_tuple = version_mod.version_tuple



from authlib.oauth2.rfc6749 import TokenMixin

class IntrospectionToken(dict[str, object], TokenMixin):
    def get_client_id(self): ...
    def get_scope(self): ...
    def get_expires_in(self): ...
    def get_expires_at(self): ...
    def __getattr__(self, key): ...



from collections.abc import Iterable

def namedtuple(typename: str, field_names: str | Iterable[str], verbose: bool = False, rename: bool = False): ...
def namedlist(typename: str, field_names: str | Iterable[str], verbose: bool = False, rename: bool = False): ...

__all__ = ["namedlist", "namedtuple"]



#!/usr/bin/env python3
"""Test typeshed's stdlib using stubtest.

stubtest is a script in the mypy project that compares stubs to the actual objects at runtime.
Note that therefore the output of stubtest depends on which Python version it is run with.
In typeshed CI, we run stubtest with each currently supported Python minor version.

"""

from __future__ import annotations

import subprocess
import sys
from pathlib import Path

from ts_utils.paths import TS_BASE_PATH, allowlists_path
from ts_utils.utils import allowlist_stubtest_arguments


def run_stubtest(typeshed_dir: Path) -> int:
    # Note when stubtest imports distutils, it will likely actually import setuptools._distutils
    # This is fine because we don't care about distutils and allowlist all errors from it
    # https://github.com/python/typeshed/pull/10253#discussion_r1216712404
    # https://github.com/python/typeshed/pull/9734
    cmd = [
        sys.executable,
        "-m",
        "mypy.stubtest",
        "--check-typeshed",
        "--show-traceback",
        "--custom-typeshed-dir",
        str(typeshed_dir),
        *allowlist_stubtest_arguments("stdlib"),
    ]
    if sys.version_info < (3, 10):
        # As discussed in https://github.com/python/typeshed/issues/3693, we only aim for
        # positional-only arg accuracy for python 3.10 and above.
        cmd += ["--ignore-positional-only"]
    print(" ".join(cmd), file=sys.stderr)
    try:
        subprocess.run(cmd, check=True)
    except subprocess.CalledProcessError as e:
        print(
            "\nNB: stubtest output depends on the Python version (and system) it is run with. "
            + "See README.md for more details.\n"
            + "NB: We only check positional-only arg accuracy for Python 3.10.\n"
            + f"\nCommand run was: {' '.join(cmd)}\n",
            file=sys.stderr,
        )
        print("\n\n", file=sys.stderr)
        print(
            f'To fix "unused allowlist" errors, remove the corresponding entries from {allowlists_path("stdlib")}',
            file=sys.stderr,
        )
        return e.returncode
    else:
        print("stubtest succeeded", file=sys.stderr)
        return 0


if __name__ == "__main__":
    sys.exit(run_stubtest(typeshed_dir=TS_BASE_PATH))



from collections.abc import Iterable
from typing import Any, Literal, TypedDict, type_check_only
from typing_extensions import TypeAlias

from docker.types import IPAMConfig

@type_check_only
class _HasId(TypedDict):
    Id: str

@type_check_only
class _HasID(TypedDict):
    ID: str

_Network: TypeAlias = _HasId | _HasID | str
_Container: TypeAlias = _HasId | _HasID | str

class NetworkApiMixin:
    def networks(self, names=None, ids=None, filters=None): ...
    def create_network(
        self,
        name: str,
        driver: str | None = None,
        options: dict[str, Any] | None = None,
        ipam: IPAMConfig | None = None,
        check_duplicate: bool | None = None,
        internal: bool = False,
        labels: dict[str, Any] | None = None,
        enable_ipv6: bool = False,
        attachable: bool | None = None,
        scope: Literal["local", "global", "swarm"] | None = None,
        ingress: bool | None = None,
    ) -> dict[str, str]: ...
    def prune_networks(self, filters=None): ...
    def remove_network(self, net_id: _Network) -> None: ...
    def inspect_network(
        self, net_id: _Network, verbose: bool | None = None, scope: Literal["local", "global", "swarm"] | None = None
    ): ...
    def connect_container_to_network(
        self,
        container: _Container,
        net_id: str,
        ipv4_address=None,
        ipv6_address=None,
        aliases=None,
        links: dict[str, str] | dict[str, None] | dict[str, str | None] | Iterable[tuple[str, str | None]] | None = None,
        link_local_ips=None,
        driver_opt=None,
        mac_address=None,
    ) -> None: ...
    def disconnect_container_from_network(self, container: _Container, net_id: str, force: bool = False) -> None: ...



from collections.abc import Sequence

def merge_flags(cfg1: dict[str, list[str]], cfg2: dict[str, list[str]]) -> dict[str, list[str]]: ...
def call(libname: str, flag: str, encoding: str = "utf-8") -> str: ...
def flags_from_pkgconfig(libs: Sequence[str]) -> dict[str, list[str]]: ...



import datetime as dt
from _typeshed import Incomplete, SupportsGetItem
from collections.abc import Callable, Iterable, Mapping, Sequence
from typing import Any, Literal, SupportsIndex, TypeVar, overload
from typing_extensions import TypeAlias, deprecated

import numpy as np
import pandas as pd
from matplotlib.axes import Axes
from matplotlib.figure import Figure
from matplotlib.legend import Legend
from matplotlib.text import Text
from matplotlib.ticker import Locator
from matplotlib.typing import ColorType
from numpy.typing import ArrayLike, NDArray
from pandas import DataFrame

from .axisgrid import Grid

__all__ = [
    "desaturate",
    "saturate",
    "set_hls_values",
    "move_legend",
    "despine",
    "get_dataset_names",
    "get_data_home",
    "load_dataset",
]

_VectorT = TypeVar("_VectorT", bound=SupportsGetItem[Any, Any])

# Type aliases used heavily throughout seaborn
_ErrorBar: TypeAlias = str | tuple[str, float] | Callable[[Iterable[float]], tuple[float, float]]  # noqa: Y047
_Estimator: TypeAlias = str | Callable[..., Incomplete]  # noqa: Y047
_Legend: TypeAlias = Literal["auto", "brief", "full"] | bool  # noqa: Y047
_LogScale: TypeAlias = bool | float | tuple[bool | float, bool | float]  # noqa: Y047
# `palette` requires dict but we use mapping to avoid a very long union because dict is invariant in its value type
_Palette: TypeAlias = str | Sequence[ColorType] | Mapping[Any, ColorType]  # noqa: Y047
_Seed: TypeAlias = int | np.random.Generator | np.random.RandomState  # noqa: Y047
_Scalar: TypeAlias = (
    # numeric
    float
    | complex
    | np.number[Any]
    # categorical
    | bool
    | str
    | bytes
    | None
    # dates
    | dt.date
    | dt.datetime
    | dt.timedelta
    | pd.Timestamp
    | pd.Timedelta
)
_Vector: TypeAlias = Iterable[_Scalar]
_DataSourceWideForm: TypeAlias = (  # noqa: Y047
    # Mapping of keys to "convertible to pd.Series" vectors
    Mapping[Any, _Vector]
    # Sequence of "convertible to pd.Series" vectors
    | Sequence[_Vector]
    # A "convertible to pd.DataFrame" table
    | Mapping[Any, Mapping[Any, _Scalar]]
    | NDArray[Any]
    # Flat "convertible to pd.Series" vector of scalars
    | Sequence[_Scalar]
)

DATASET_SOURCE: str
DATASET_NAMES_URL: str

def ci_to_errsize(cis: ArrayLike, heights: ArrayLike) -> NDArray[np.float64]: ...
def desaturate(color: ColorType, prop: float) -> tuple[float, float, float]: ...
def saturate(color: ColorType) -> tuple[float, float, float]: ...
def set_hls_values(
    color: ColorType, h: float | None = None, l: float | None = None, s: float | None = None
) -> tuple[float, float, float]: ...
@deprecated("Function `axlabel` is deprecated and will be removed in a future version")
def axlabel(xlabel: str, ylabel: str, **kwargs: Any) -> None: ...
def remove_na(vector: _VectorT) -> _VectorT: ...
def get_color_cycle() -> list[str]: ...

# `despine` should be kept roughly in line with `seaborn.axisgrid.FacetGrid.despine`
def despine(
    fig: Figure | None = None,
    ax: Axes | None = None,
    top: bool = True,
    right: bool = True,
    left: bool = False,
    bottom: bool = False,
    offset: int | Mapping[str, int] | None = None,
    trim: bool = False,
) -> None: ...
def move_legend(obj: Grid | Axes | Figure, loc: str | int, **kwargs: Any) -> None: ...
def ci(
    a: float | ArrayLike, which: float | ArrayLike = 95, axis: SupportsIndex | Sequence[SupportsIndex] | None = None
) -> NDArray[np.float64]: ...
def get_dataset_names() -> list[str]: ...
def get_data_home(data_home: str | None = None) -> str: ...
def load_dataset(name: str, cache: bool = True, data_home: str | None = None, **kws: Any) -> DataFrame: ...
def axis_ticklabels_overlap(labels: Iterable[Text]) -> bool: ...
def axes_ticklabels_overlap(ax: Axes) -> tuple[bool, bool]: ...
def locator_to_legend_entries(locator: Locator, limits: Iterable[float], dtype) -> tuple[list[Incomplete], list[str]]: ...
@overload
def relative_luminance(color: ColorType) -> float: ...  # type: ignore[overload-overlap]
@overload
def relative_luminance(color: Sequence[ColorType]) -> NDArray[np.float64]: ...
@overload
def relative_luminance(color: ColorType | Sequence[ColorType] | ArrayLike) -> float | NDArray[np.float64]: ...
def to_utf8(obj: object) -> str: ...
def adjust_legend_subtitles(legend: Legend) -> None: ...  # not public API



import datetime
from typing import ClassVar

from ..core import WesternCalendar

class Germany(WesternCalendar):
    all_time_include_reformation_day: ClassVar[bool]
    include_reformation_day_2018: ClassVar[bool]
    def include_reformation_day(self, year: int) -> bool: ...
    def get_reformation_day(self, year: int) -> tuple[datetime.date, str]: ...

class BadenWurttemberg(Germany): ...
class Bavaria(Germany): ...

class Berlin(Germany):
    def get_international_womens_day(self, year: int) -> tuple[datetime.date, str]: ...
    def get_liberation_day(self, year: int) -> tuple[datetime.date, str]: ...

class Brandenburg(Germany): ...
class Bremen(Germany): ...
class Hamburg(Germany): ...
class Hesse(Germany): ...
class MecklenburgVorpommern(Germany): ...
class LowerSaxony(Germany): ...
class NorthRhineWestphalia(Germany): ...
class RhinelandPalatinate(Germany): ...
class Saarland(Germany): ...

class Saxony(Germany):
    def get_repentance_day(self, year: int) -> tuple[datetime.date, str]: ...

class SaxonyAnhalt(Germany): ...
class SchleswigHolstein(Germany): ...
class Thuringia(Germany): ...



rdates_property: property
exdates_property: property
rrules_property: property

def multi_language_text_property(main_prop: str, compatibility_prop: str, doc: str) -> property: ...
def single_int_property(prop: str, default: int, doc: str) -> property: ...
def single_utc_property(name: str, docs: str) -> property: ...
def single_string_property(name: str, docs: str, other_name: str | None = None) -> property: ...

color_property: property
sequence_property: property
categories_property: property
uid_property: property

__all__ = [
    "categories_property",
    "color_property",
    "exdates_property",
    "multi_language_text_property",
    "rdates_property",
    "rrules_property",
    "sequence_property",
    "single_int_property",
    "single_utc_property",
    "uid_property",
]



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class VersionDef(google.protobuf.message.Message):
    """This file is a copy of the TensorFlow Versions proto.
    Keep this file in sync with the source proto definition at
    https://github.com/tensorflow/tensorflow/blob/master/tensorflow/core/framework/versions.proto

    Version information for a piece of serialized data

    There are different types of versions for each type of data
    (GraphDef, etc.), but they all have the same common shape
    described here.

    Each consumer has "consumer" and "min_producer" versions (specified
    elsewhere).  A consumer is allowed to consume this data if

      producer >= min_producer
      consumer >= min_consumer
      consumer not in bad_consumers

    LINT.IfChange
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PRODUCER_FIELD_NUMBER: builtins.int
    MIN_CONSUMER_FIELD_NUMBER: builtins.int
    BAD_CONSUMERS_FIELD_NUMBER: builtins.int
    producer: builtins.int
    """The version of the code that produced this data."""
    min_consumer: builtins.int
    """Any consumer below this version is not allowed to consume this data."""
    @property
    def bad_consumers(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
        """Specific consumer versions which are disallowed (e.g. due to bugs)."""

    def __init__(
        self,
        *,
        producer: builtins.int | None = ...,
        min_consumer: builtins.int | None = ...,
        bad_consumers: collections.abc.Iterable[builtins.int] | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal["bad_consumers", b"bad_consumers", "min_consumer", b"min_consumer", "producer", b"producer"],
    ) -> None: ...

global___VersionDef = VersionDef



from _typeshed import Incomplete
from typing import ClassVar

class Key:
    kty: str
    ALLOWED_PARAMS: ClassVar[list[str]]
    PRIVATE_KEY_OPS: ClassVar[list[str]]
    PUBLIC_KEY_OPS: ClassVar[list[str]]
    REQUIRED_JSON_FIELDS: ClassVar[list[str]]
    options: Incomplete
    def __init__(self, options=None) -> None: ...
    @property
    def tokens(self): ...
    @property
    def kid(self): ...
    def keys(self): ...
    def __getitem__(self, item): ...
    @property
    def public_only(self) -> None: ...
    def load_raw_key(self) -> None: ...
    def load_dict_key(self) -> None: ...
    def check_key_op(self, operation) -> None: ...
    def as_dict(self, is_private: bool = False, **params) -> None: ...
    def as_json(self, is_private: bool = False, **params): ...
    def thumbprint(self): ...
    @classmethod
    def check_required_fields(cls, data) -> None: ...
    @classmethod
    def validate_raw_key(cls, key) -> None: ...



from _typeshed import Incomplete
from datetime import datetime
from typing import NamedTuple

from .actions import Action
from .enums import AnnotationFlag, AnnotationName, FileAttachmentAnnotationName
from .syntax import Destination, Name, PDFContentStream, PDFObject

DEFAULT_ANNOT_FLAGS: tuple[AnnotationFlag, ...]

class AnnotationMixin:
    type: Name
    subtype: Name
    rect: str
    border: str
    f_t: Name | None
    v: Incomplete | None
    f: int  # AnnotationFlags bitmask
    contents: str | None
    a: Action | None
    dest: Destination | None
    c: str | None
    t: str | None
    m: str | None
    quad_points: str | None
    p: Incomplete | None
    name: AnnotationName | FileAttachmentAnnotationName | None
    ink_list: str | None
    f_s: str | None
    d_a: str | None
    def __init__(
        self,
        subtype: str,
        x: int,
        y: int,
        width: int,
        height: int,
        flags: tuple[AnnotationFlag | str, ...] = ...,
        contents: str | None = None,
        dest: Destination | None = None,
        action: Action | None = None,
        color: tuple[int, int, int] | None = None,
        modification_time: datetime | None = None,
        title: str | None = None,
        quad_points: tuple[float, ...] | None = None,  # multiple of 8 floats
        border_width: int = 0,
        name: AnnotationName | FileAttachmentAnnotationName | None = None,
        ink_list: tuple[int, ...] = (),
        file_spec: str | None = None,
        field_type: str | None = None,
        value=None,
        default_appearance: str | None = None,
    ) -> None: ...

class PDFAnnotation(AnnotationMixin, PDFObject): ...

class AnnotationDict(AnnotationMixin):
    __slots__ = (
        "type",
        "subtype",
        "rect",
        "border",
        "f_t",
        "v",
        "f",
        "contents",
        "a",
        "dest",
        "c",
        "t",
        "quad_points",
        "p",
        "name",
        "ink_list",
        "f_s",
        "d_a",
    )
    def serialize(self) -> str: ...

class PDFEmbeddedFile(PDFContentStream):
    type: Name
    params: str
    def __init__(
        self,
        basename: str,
        contents: bytes,
        desc: str = "",
        creation_date: datetime | None = None,
        modification_date: datetime | None = None,
        compress: bool = False,
        checksum: bool = False,
    ) -> None: ...
    def globally_enclosed(self) -> bool: ...
    def set_globally_enclosed(self, value: bool) -> None: ...
    def basename(self) -> str: ...
    def file_spec(self) -> FileSpec: ...

class FileSpec(NamedTuple):
    embedded_file: PDFEmbeddedFile
    basename: str
    desc: str
    def serialize(self) -> str: ...



from _typeshed import StrOrBytesPath
from abc import abstractmethod
from logging import Logger
from subprocess import Popen
from typing import Any, ClassVar, Literal, NewType

import setuptools.command.sdist
from fanstatic.core import Resource

logger: Logger

class CompilerError(Exception): ...

class Compiler:
    @property
    @abstractmethod
    def name(self) -> str: ...
    @property
    @abstractmethod
    def source_extension(self) -> str: ...
    def __call__(self, resource: Resource, force: bool = False) -> None: ...
    def process(self, source: StrOrBytesPath, target: StrOrBytesPath) -> Any: ...
    def should_process(self, source: StrOrBytesPath, target: StrOrBytesPath) -> bool: ...
    @property
    @abstractmethod
    def available(self) -> bool: ...
    def source_path(self, resource: Resource) -> str | None: ...
    def target_path(self, resource: Resource) -> str | None: ...

class Minifier(Compiler):
    @property
    @abstractmethod
    def name(self) -> str: ...
    @property
    @abstractmethod
    def source_extension(self) -> str: ...
    @property
    @abstractmethod
    def target_extension(self) -> str: ...
    def source_to_target(self, resource: Resource) -> str: ...

def compile_resources(argv: list[str] = ...) -> None: ...

class sdist_compile(setuptools.command.sdist.sdist): ...

class NullCompiler(Compiler):
    name: ClassVar[Literal[""]]
    source_extension = NotImplemented
    def source_path(self, resource: Resource) -> None: ...
    def target_path(self, resource: Resource) -> None: ...
    def should_process(self, source: StrOrBytesPath, target: StrOrBytesPath) -> Literal[False]: ...
    @property
    def available(self) -> Literal[False]: ...

_SourceType = NewType("_SourceType", object)
_TargetType = NewType("_TargetType", object)
SOURCE: _SourceType
TARGET: _TargetType

class CommandlineBase:
    @property
    @abstractmethod
    def command(self) -> str: ...
    arguments: ClassVar[list[str]]
    @property
    def available(self) -> bool: ...
    def process(self, source: StrOrBytesPath | _SourceType, target: StrOrBytesPath | _TargetType) -> Popen[str]: ...

class CoffeeScript(CommandlineBase, Compiler):
    name: ClassVar[Literal["coffee"]]
    command: ClassVar[Literal["coffee"]]
    source_extension = NotImplemented
    def process(  # type: ignore[override]
        self, source: StrOrBytesPath | _SourceType, target: StrOrBytesPath | _TargetType
    ) -> None: ...

COFFEE_COMPILER: CoffeeScript

class LESS(CommandlineBase, Compiler):
    name: ClassVar[Literal["less"]]
    command: ClassVar[Literal["lessc"]]
    source_extension = NotImplemented

LESS_COMPILER: LESS

class SASS(CommandlineBase, Compiler):
    name: ClassVar[Literal["sass"]]
    command: ClassVar[Literal["sass"]]
    source_extension: ClassVar[Literal[".scss"]]

SASS_COMPILER: SASS

class PythonPackageBase:
    @property
    @abstractmethod
    def package(self) -> str: ...
    @property
    def available(self) -> bool: ...

class CSSMin(PythonPackageBase, Minifier):
    name: ClassVar[Literal["cssmin"]]
    package: ClassVar[Literal["cssmin"]]
    source_extension = NotImplemented
    target_extension: ClassVar[Literal[".min.css"]]

CSSMIN_MINIFIER: CSSMin

class JSMin(PythonPackageBase, Minifier):
    name: ClassVar[Literal["jsmin"]]
    package: ClassVar[Literal["jsmin"]]
    source_extension = NotImplemented
    target_extension: ClassVar[Literal[".min.js"]]

JSMIN_MINIFIER: JSMin

class Closure(PythonPackageBase, Minifier):
    name: ClassVar[Literal["closure"]]
    package: ClassVar[Literal["closure"]]
    source_extension = NotImplemented
    target_extension: Literal[".min.js"]
    arguments: ClassVar[list[str]]
    def process(self, source: StrOrBytesPath, target: StrOrBytesPath) -> Popen[str]: ...

CLOSURE_MINIFIER: Closure



from logging import Logger

from aws_xray_sdk.ext.dbapi2 import XRayTracedCursor

log: Logger

def patch_db() -> None: ...

class DjangoXRayTracedCursor(XRayTracedCursor):
    def execute(self, query, *args, **kwargs): ...
    def executemany(self, query, *args, **kwargs): ...
    def callproc(self, proc, args): ...



import datetime

from pyasn1.type import char
from pyasn1.type.tag import TagSet

__all__ = ["ObjectDescriptor", "GeneralizedTime", "UTCTime"]

class ObjectDescriptor(char.GraphicString):
    tagSet: TagSet
    typeId: int

class TimeMixIn:
    class FixedOffset(datetime.tzinfo):
        def __init__(self, offset: int = 0, name: str = "UTC") -> None: ...
        def utcoffset(self, dt): ...
        def tzname(self, dt): ...
        def dst(self, dt): ...

    UTC: FixedOffset
    @property
    def asDateTime(self): ...
    @classmethod
    def fromDateTime(cls, dt): ...

class GeneralizedTime(char.VisibleString, TimeMixIn):
    tagSet: TagSet
    typeId: int

class UTCTime(char.VisibleString, TimeMixIn):
    tagSet: TagSet
    typeId: int



from _typeshed import Incomplete
from collections.abc import Generator, Iterable
from typing import Final, Literal

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["edge_dfs"]

FORWARD: Final = "forward"
REVERSE: Final = "reverse"

@_dispatchable
def edge_dfs(
    G: Graph[_Node],
    source: _Node | Iterable[_Node] | None = None,
    orientation: Literal["original", "reverse", "ignore"] | None = None,
) -> Generator[tuple[Incomplete, ...]]: ...



from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable
from numpy.random import RandomState

__all__ = ["leiden_communities", "leiden_partitions"]

@_dispatchable
def leiden_communities(
    G: Graph[_Node],
    weight: str | None = "weight",
    resolution: float = 1,
    max_level: int | None = None,
    seed: int | RandomState | None = None,
): ...
@_dispatchable
def leiden_partitions(
    G: Graph[_Node], weight: str | None = "weight", resolution: float = 1, seed: int | RandomState | None = None
): ...



# This module is made specifically to abstract away those type errors
# pyright: reportUnknownVariableType=false, reportUnknownArgumentType=false

"""Tools to help parse and validate information stored in METADATA.toml files."""

from __future__ import annotations

import datetime
import functools
import re
import sys
import urllib.parse
from collections.abc import Mapping
from dataclasses import dataclass
from pathlib import Path
from typing import Annotated, Any, Final, NamedTuple, final
from typing_extensions import TypeGuard

if sys.version_info >= (3, 11):
    import tomllib
else:
    import tomli as tomllib

import tomlkit
from packaging.requirements import Requirement
from packaging.specifiers import Specifier
from tomlkit.items import String

from .paths import PYPROJECT_PATH, STUBS_PATH, distribution_path

__all__ = [
    "NoSuchStubError",
    "PackageDependencies",
    "StubMetadata",
    "StubtestSettings",
    "get_oldest_supported_python",
    "get_recursive_requirements",
    "read_dependencies",
    "read_metadata",
    "read_stubtest_settings",
]

DEFAULT_STUBTEST_PLATFORMS = ["linux"]

_STUBTEST_PLATFORM_MAPPING: Final = {"linux": "apt_dependencies", "darwin": "brew_dependencies", "win32": "choco_dependencies"}
# Some older websites have a bad pattern of using query params for navigation.
_QUERY_URL_ALLOWLIST = {"sourceware.org"}


def _is_list_of_strings(obj: object) -> TypeGuard[list[str]]:
    return isinstance(obj, list) and all(isinstance(item, str) for item in obj)


def _is_nested_dict(obj: object) -> TypeGuard[dict[str, dict[str, Any]]]:
    return isinstance(obj, dict) and all(isinstance(k, str) and isinstance(v, dict) for k, v in obj.items())


@functools.cache
def get_oldest_supported_python() -> str:
    with PYPROJECT_PATH.open("rb") as config:
        val = tomllib.load(config)["tool"]["typeshed"]["oldest_supported_python"]
    assert type(val) is str
    return val


def metadata_path(distribution: str) -> Path:
    """Return the path to the METADATA.toml file of a third-party distribution."""
    return distribution_path(distribution) / "METADATA.toml"


@final
@dataclass(frozen=True)
class StubtestSettings:
    """The stubtest settings for a single stubs distribution.

    Don't construct instances directly; use the `read_stubtest_settings` function.
    """

    skip: bool
    apt_dependencies: list[str]
    brew_dependencies: list[str]
    choco_dependencies: list[str]
    extras: list[str]
    ignore_missing_stub: bool
    supported_platforms: list[str] | None  # None means all platforms
    ci_platforms: list[str]
    stubtest_requirements: list[str]
    mypy_plugins: list[str]
    mypy_plugins_config: dict[str, dict[str, Any]]

    def system_requirements_for_platform(self, platform: str) -> list[str]:
        assert platform in _STUBTEST_PLATFORM_MAPPING, f"Unrecognised platform {platform!r}"
        ret = getattr(self, _STUBTEST_PLATFORM_MAPPING[platform])
        assert _is_list_of_strings(ret)
        return ret


@functools.cache
def read_stubtest_settings(distribution: str) -> StubtestSettings:
    """Return an object describing the stubtest settings for a single stubs distribution."""
    with metadata_path(distribution).open("rb") as f:
        data: dict[str, object] = tomllib.load(f).get("tool", {}).get("stubtest", {})

    skip: object = data.get("skip", False)
    apt_dependencies: object = data.get("apt_dependencies", [])
    brew_dependencies: object = data.get("brew_dependencies", [])
    choco_dependencies: object = data.get("choco_dependencies", [])
    extras: object = data.get("extras", [])
    ignore_missing_stub: object = data.get("ignore_missing_stub", False)
    supported_platforms: object = data.get("supported_platforms")
    ci_platforms: object = data.get("ci_platforms", DEFAULT_STUBTEST_PLATFORMS)
    stubtest_requirements: object = data.get("stubtest_requirements", [])
    mypy_plugins: object = data.get("mypy_plugins", [])
    mypy_plugins_config: object = data.get("mypy_plugins_config", {})

    assert type(skip) is bool
    assert type(ignore_missing_stub) is bool

    # It doesn't work for type-narrowing if we use a for loop here...
    assert supported_platforms is None or _is_list_of_strings(supported_platforms)
    assert _is_list_of_strings(ci_platforms)
    assert _is_list_of_strings(apt_dependencies)
    assert _is_list_of_strings(brew_dependencies)
    assert _is_list_of_strings(choco_dependencies)
    assert _is_list_of_strings(extras)
    assert _is_list_of_strings(stubtest_requirements)
    assert _is_list_of_strings(mypy_plugins)
    assert _is_nested_dict(mypy_plugins_config)

    unrecognised_platforms = set(ci_platforms) - _STUBTEST_PLATFORM_MAPPING.keys()
    assert not unrecognised_platforms, f"Unrecognised ci_platforms specified for {distribution!r}: {unrecognised_platforms}"

    if supported_platforms is not None:
        assert set(ci_platforms).issubset(
            supported_platforms
        ), f"ci_platforms must be a subset of supported_platforms for {distribution!r}"

    for platform, dep_key in _STUBTEST_PLATFORM_MAPPING.items():
        if platform not in ci_platforms:
            assert dep_key not in data, (
                f"Stubtest is not run on {platform} in CI for {distribution!r}, "
                f"but {dep_key!r} are specified in METADATA.toml"
            )

    return StubtestSettings(
        skip=skip,
        apt_dependencies=apt_dependencies,
        brew_dependencies=brew_dependencies,
        choco_dependencies=choco_dependencies,
        extras=extras,
        ignore_missing_stub=ignore_missing_stub,
        supported_platforms=supported_platforms,
        ci_platforms=ci_platforms,
        stubtest_requirements=stubtest_requirements,
        mypy_plugins=mypy_plugins,
        mypy_plugins_config=mypy_plugins_config,
    )


@final
@dataclass(frozen=True)
class ObsoleteMetadata:
    since_version: Annotated[str, "A string representing a specific version"]
    since_date: Annotated[datetime.date, "A date when the package became obsolete"]


@final
@dataclass(frozen=True)
class StubMetadata:
    """The metadata for a single stubs distribution.

    Don't construct instances directly; use the `read_metadata` function.
    """

    distribution: Annotated[str, "The name of the distribution on PyPI"]
    version_spec: Annotated[Specifier, "Upstream versions that the stubs are compatible with"]
    requires: Annotated[list[Requirement], "The parsed requirements as listed in METADATA.toml"]
    extra_description: str | None
    stub_distribution: Annotated[str, "The name under which the distribution is uploaded to PyPI"]
    upstream_repository: Annotated[str, "The URL of the upstream repository"] | None
    obsolete: Annotated[ObsoleteMetadata, "Metadata indicating when the stubs package became obsolete"] | None
    no_longer_updated: bool
    uploaded_to_pypi: Annotated[bool, "Whether or not a distribution is uploaded to PyPI"]
    partial_stub: Annotated[bool, "Whether this is a partial type stub package as per PEP 561."]
    stubtest_settings: StubtestSettings
    requires_python: Annotated[Specifier, "Versions of Python supported by the stub package"]

    @property
    def is_obsolete(self) -> bool:
        return self.obsolete is not None


_KNOWN_METADATA_FIELDS: Final = frozenset(
    {
        "version",
        "requires",
        "extra_description",
        "stub_distribution",
        "upstream_repository",
        "obsolete_since",
        "no_longer_updated",
        "upload",
        "tool",
        "partial_stub",
        "requires_python",
        "mypy-tests",
    }
)
_KNOWN_METADATA_TOOL_FIELDS: Final = {
    "stubtest": {
        "skip",
        "apt_dependencies",
        "brew_dependencies",
        "choco_dependencies",
        "extras",
        "ignore_missing_stub",
        "supported_platforms",
        "ci_platforms",
        "stubtest_requirements",
        "mypy_plugins",
        "mypy_plugins_config",
    }
}
_DIST_NAME_RE: Final = re.compile(r"^[a-z0-9]([a-z0-9._-]*[a-z0-9])?$", re.IGNORECASE)


class NoSuchStubError(ValueError):
    """Raise NoSuchStubError to indicate that a stubs/{distribution} directory doesn't exist."""


@functools.cache
def read_metadata(distribution: str) -> StubMetadata:
    """Return an object describing the metadata of a stub as given in the METADATA.toml file.

    This function does some basic validation,
    but does no parsing, transforming or normalization of the metadata.
    Use `read_dependencies` if you need to parse the dependencies
    given in the `requires` field, for example.
    """
    try:
        with metadata_path(distribution).open("rb") as f:
            data = tomlkit.load(f)
    except FileNotFoundError:
        raise NoSuchStubError(f"Typeshed has no stubs for {distribution!r}!") from None

    unknown_metadata_fields = data.keys() - _KNOWN_METADATA_FIELDS
    assert not unknown_metadata_fields, f"Unexpected keys in METADATA.toml for {distribution!r}: {unknown_metadata_fields}"

    assert "version" in data, f"Missing 'version' field in METADATA.toml for {distribution!r}"
    version: object = data.get("version")  # pyright: ignore[reportUnknownMemberType]
    assert isinstance(version, str) and len(version) > 0, f"Invalid 'version' field in METADATA.toml for {distribution!r}"
    # Check that the version spec parses
    if version[0].isdigit():
        version = f"=={version}"
    version_spec = Specifier(version)
    assert version_spec.operator in {"==", "~="}, f"Invalid 'version' field in METADATA.toml for {distribution!r}"

    requires_s: object = data.get("requires", [])  # pyright: ignore[reportUnknownMemberType]
    assert isinstance(requires_s, list)
    requires = [parse_requires(distribution, req) for req in requires_s]

    extra_description: object = data.get("extra_description")  # pyright: ignore[reportUnknownMemberType]
    assert isinstance(extra_description, (str, type(None)))

    if "stub_distribution" in data:
        stub_distribution = data["stub_distribution"]
        assert isinstance(stub_distribution, str)
        assert _DIST_NAME_RE.fullmatch(stub_distribution), f"Invalid 'stub_distribution' value for {distribution!r}"
    else:
        stub_distribution = f"types-{distribution}"

    upstream_repository: object = data.get("upstream_repository")  # pyright: ignore[reportUnknownMemberType]
    assert isinstance(upstream_repository, (str, type(None)))
    if isinstance(upstream_repository, str):
        parsed_url = urllib.parse.urlsplit(upstream_repository)
        assert parsed_url.scheme == "https", f"{distribution}: URLs in the upstream_repository field should use https"
        no_www_please = (
            f"{distribution}: `World Wide Web` subdomain (`www.`) should be removed from URLs in the upstream_repository field"
        )
        assert not parsed_url.netloc.startswith("www."), no_www_please
        no_query_params_please = (
            f"{distribution}: Query params (`?`) should be removed from URLs in the upstream_repository field"
        )
        assert parsed_url.hostname in _QUERY_URL_ALLOWLIST or (not parsed_url.query), no_query_params_please
        no_fragments_please = f"{distribution}: Fragments (`#`) should be removed from URLs in the upstream_repository field"
        assert not parsed_url.fragment, no_fragments_please
        if parsed_url.netloc == "github.com":
            cleaned_url_path = parsed_url.path.strip("/")
            num_url_path_parts = len(Path(cleaned_url_path).parts)
            bad_github_url_msg = (
                f"Invalid upstream_repository for {distribution!r}: "
                "URLs for GitHub repositories always have two parts in their paths"
            )
            assert num_url_path_parts == 2, bad_github_url_msg

    obsolete_since: object = data.get("obsolete_since")  # pyright: ignore[reportUnknownMemberType]
    assert isinstance(obsolete_since, (String, type(None)))
    if obsolete_since:
        comment = obsolete_since.trivia.comment
        since_date_string = comment.removeprefix("# Released on ")
        since_date = datetime.date.fromisoformat(since_date_string)
        obsolete = ObsoleteMetadata(since_version=obsolete_since, since_date=since_date)
    else:
        obsolete = None
    no_longer_updated: object = data.get("no_longer_updated", False)  # pyright: ignore[reportUnknownMemberType]
    assert type(no_longer_updated) is bool
    uploaded_to_pypi: object = data.get("upload", True)  # pyright: ignore[reportUnknownMemberType]
    assert type(uploaded_to_pypi) is bool
    partial_stub: object = data.get("partial_stub", True)  # pyright: ignore[reportUnknownMemberType]
    assert type(partial_stub) is bool
    requires_python_str: object = data.get("requires_python")  # pyright: ignore[reportUnknownMemberType]
    oldest_supported_python = get_oldest_supported_python()
    oldest_supported_python_specifier = Specifier(f">={oldest_supported_python}")
    if requires_python_str is None:
        requires_python = oldest_supported_python_specifier
    else:
        assert isinstance(requires_python_str, str)
        requires_python = Specifier(requires_python_str)
        assert requires_python != oldest_supported_python_specifier, f'requires_python="{requires_python}" is redundant'
        # Check minimum Python version is not less than the oldest version of Python supported by typeshed
        assert oldest_supported_python_specifier.contains(
            requires_python.version
        ), f"'requires_python' contains versions lower than typeshed's oldest supported Python ({oldest_supported_python})"
        assert requires_python.operator == ">=", "'requires_python' should be a minimum version specifier, use '>=3.x'"

    empty_tools: dict[object, object] = {}
    tools_settings: object = data.get("tool", empty_tools)  # pyright: ignore[reportUnknownMemberType]
    assert isinstance(tools_settings, dict)
    assert tools_settings.keys() <= _KNOWN_METADATA_TOOL_FIELDS.keys(), f"Unrecognised tool for {distribution!r}"
    for tool, tk in _KNOWN_METADATA_TOOL_FIELDS.items():
        settings_for_tool: object = tools_settings.get(tool, {})  # pyright: ignore[reportUnknownMemberType]
        assert isinstance(settings_for_tool, dict)
        for key in settings_for_tool:
            assert key in tk, f"Unrecognised {tool} key {key!r} for {distribution!r}"

    return StubMetadata(
        distribution=distribution,
        version_spec=version_spec,
        requires=requires,
        extra_description=extra_description,
        stub_distribution=stub_distribution,
        upstream_repository=upstream_repository,
        obsolete=obsolete,
        no_longer_updated=no_longer_updated,
        uploaded_to_pypi=uploaded_to_pypi,
        partial_stub=partial_stub,
        stubtest_settings=read_stubtest_settings(distribution),
        requires_python=requires_python,
    )


def update_metadata(distribution: str, **new_values: object) -> tomlkit.TOMLDocument:
    """Update a distribution's METADATA.toml.

    Return the updated TOML dictionary for use without having to open the file separately.
    """
    path = metadata_path(distribution)
    try:
        with path.open("rb") as file:
            data = tomlkit.load(file)
    except FileNotFoundError:
        raise NoSuchStubError(f"Typeshed has no stubs for {distribution!r}!") from None
    data.update(new_values)  # pyright: ignore[reportUnknownMemberType] # tomlkit.TOMLDocument.update is partially typed
    with path.open("w", encoding="UTF-8") as file:
        tomlkit.dump(data, file)  # pyright: ignore[reportUnknownMemberType] # tomlkit.dump has partially unknown Mapping type
    return data


def parse_requires(distribution: str, req: object) -> Requirement:
    assert isinstance(req, str), f"Invalid requirement {req!r} for {distribution!r}"
    return Requirement(req)


class PackageDependencies(NamedTuple):
    typeshed_pkgs: tuple[Requirement, ...]
    external_pkgs: tuple[Requirement, ...]


@functools.cache
def get_pypi_name_to_typeshed_name_mapping() -> Mapping[str, str]:
    return {read_metadata(stub_dir.name).stub_distribution: stub_dir.name for stub_dir in STUBS_PATH.iterdir()}


@functools.cache
def read_dependencies(distribution: str) -> PackageDependencies:
    """Read the dependencies listed in a METADATA.toml file for a stubs package.

    Once the dependencies have been read,
    determine which dependencies are typeshed-internal dependencies,
    and which dependencies are external (non-types) dependencies.
    For typeshed dependencies, translate the "dependency name" into the "package name";
    for external dependencies, leave them as they are in the METADATA.toml file.

    Note that this function may consider things to be typeshed stubs
    even if they haven't yet been uploaded to PyPI.
    If a typeshed stub is removed, this function will consider it to be an external dependency.
    """
    pypi_name_to_typeshed_name_mapping = get_pypi_name_to_typeshed_name_mapping()
    typeshed: list[Requirement] = []
    external: list[Requirement] = []
    for dependency in read_metadata(distribution).requires:
        if dependency.name in pypi_name_to_typeshed_name_mapping:
            req = Requirement(str(dependency))  # copy the requirement
            req.name = pypi_name_to_typeshed_name_mapping[dependency.name]
            typeshed.append(req)
        else:
            external.append(dependency)
    return PackageDependencies(tuple(typeshed), tuple(external))


@functools.cache
def get_recursive_requirements(package_name: str) -> PackageDependencies:
    """Recursively gather dependencies for a single stubs package.

    For example, if the stubs for `caldav`
    declare a dependency on typeshed's stubs for `requests`,
    and the stubs for requests declare a dependency on typeshed's stubs for `urllib3`,
    `get_recursive_requirements("caldav")` will determine that the stubs for `caldav`
    have both `requests` and `urllib3` as typeshed-internal dependencies.
    """
    typeshed: set[Requirement] = set()
    external: set[Requirement] = set()
    non_recursive_requirements = read_dependencies(package_name)
    typeshed.update(non_recursive_requirements.typeshed_pkgs)
    external.update(non_recursive_requirements.external_pkgs)
    for pkg in non_recursive_requirements.typeshed_pkgs:
        reqs = get_recursive_requirements(pkg.name)
        typeshed.update(reqs.typeshed_pkgs)
        external.update(reqs.external_pkgs)
    return PackageDependencies(tuple(typeshed), tuple(external))



import io
import json
import os
from _typeshed import Incomplete, SupportsRead
from collections.abc import Callable, Hashable
from typing import Any, Literal, final, overload
from typing_extensions import Self

import pandas as pd
from numpy.typing import ArrayLike
from pandas._typing import Axes, Dtype
from pyproj import CRS
from shapely.geometry.base import BaseGeometry

from ._decorator import doc
from .array import GeometryArray
from .base import GeoPandasBase, _BboxLike, _ClipMask, _ConvertibleToCRS, _ConvertibleToGeoSeries, _MaskLike
from .explore import _explore_geoseries
from .io._geoarrow import GeoArrowArray
from .plotting import plot_series

class GeoSeries(GeoPandasBase, pd.Series[BaseGeometry]):  # type: ignore[type-var,misc]  # pyright: ignore[reportInvalidTypeArguments]
    # Override the weird annotation of Series.__new__ in pandas-stubs
    def __new__(
        self,
        data: _ConvertibleToGeoSeries | None = None,
        index: Axes | None = None,
        crs: _ConvertibleToCRS | None = None,
        *,
        dtype: Dtype | None = None,
        name: Hashable = None,
        copy: bool | None = None,
        fastpath: bool = False,
    ) -> Self: ...
    def __init__(
        self,
        data: _ConvertibleToGeoSeries | None = None,
        index: Axes | None = None,
        crs: _ConvertibleToCRS | None = None,
        *,
        dtype: Dtype | None = None,
        name: Hashable = None,
        copy: bool | None = None,
        fastpath: bool = False,
    ) -> None: ...
    @final  # type: ignore[misc]
    def copy(self, deep: bool = True) -> Self: ...
    @property
    def values(self) -> GeometryArray: ...
    @property
    def geometry(self) -> Self: ...
    @property
    def x(self) -> pd.Series[float]: ...
    @property
    def y(self) -> pd.Series[float]: ...
    @property
    def z(self) -> pd.Series[float]: ...
    @property
    def m(self) -> pd.Series[float]: ...
    # Keep inline with GeoDataFrame.from_file and geopandas.io.file._read_file
    @classmethod
    def from_file(
        cls,
        filename: str | os.PathLike[str] | SupportsRead[Incomplete],
        *,
        bbox: _BboxLike | None = None,
        mask: _MaskLike | None = None,
        rows: int | slice | None = None,
        engine: Literal["fiona", "pyogrio"] | None = None,
        ignore_geometry: Literal[False] = False,
        layer: int | str | None = None,
        encoding: str | None = None,
        **kwargs,  # engine dependent
    ) -> GeoSeries: ...
    @classmethod
    def from_wkb(
        cls,
        data: ArrayLike,  # array-like of bytes handled by shapely.from_wkb(data)
        index: Axes | None = None,
        crs: _ConvertibleToCRS | None = None,
        on_invalid: Literal["raise", "warn", "ignore", "fix"] = "raise",
        *,
        dtype: Dtype | None = None,
        name: Hashable = None,
        copy: bool | None = None,
        fastpath: bool = False,
    ) -> Self: ...
    @classmethod
    def from_wkt(
        cls,
        data: ArrayLike,  # array-like of str handled by shapely.from_wkt(data)
        index: Axes | None = None,
        crs: _ConvertibleToCRS | None = None,
        on_invalid: Literal["raise", "warn", "ignore", "fix"] = "raise",
        *,
        dtype: Dtype | None = None,
        name: Hashable = None,
        copy: bool | None = None,
        fastpath: bool = False,
    ) -> Self: ...
    @classmethod
    def from_xy(
        cls,
        # x, y, z: array-like of floats handled by np.asarray(..., dtype="float64")
        x: ArrayLike,
        y: ArrayLike,
        z: ArrayLike | None = None,
        index: Axes | None = None,
        crs: _ConvertibleToCRS | None = None,
        *,
        dtype: Dtype | None = None,
        name: Hashable = None,
        copy: bool | None = None,
        fastpath: bool = False,
    ) -> Self: ...
    @classmethod
    def from_arrow(
        cls,
        arr,
        *,
        # GeoSeries constructor kwargs
        index: Axes | None = None,
        crs: _ConvertibleToCRS | None = None,
        dtype: Dtype | None = None,
        name: Hashable = None,
        copy: bool | None = None,
        fastpath: bool = False,
    ) -> Self: ...
    @property
    def __geo_interface__(self) -> dict[str, Any]: ...  # values are arbitrary
    # Keep method to_file roughly in line with GeoDataFrame.to_file
    def to_file(
        self,
        filename: str | os.PathLike[str] | io.BytesIO,
        driver: str | None = None,
        index: bool | None = None,
        *,
        # kwargs from `_to_file` function
        schema: dict[str, Incomplete] | None = None,
        mode: Literal["w", "a"] = "w",
        crs: _ConvertibleToCRS | None = None,
        engine: Literal["fiona", "pyogrio"] | None = None,
        metadata: dict[str, str] | None = None,
        # kwargs extracted from engines
        layer: int | str | None = None,
        encoding: str | None = None,
        overwrite: bool | None = ...,
        **kwargs,  # engine and driver dependent
    ) -> None: ...
    # *** `__getitem__`, `sort_index` and `take` are annotated with `-> Self` in pandas-stubs; no need to override them ***
    # *** `apply` annotation in pandas-stubs is compatible except for deprecated `convert_dtype` argument ***
    # def apply(self, func, convert_dtype: bool | None = None, args=(), **kwargs): ...
    def isna(self) -> pd.Series[bool]: ...
    def isnull(self) -> pd.Series[bool]: ...
    def notna(self) -> pd.Series[bool]: ...
    def notnull(self) -> pd.Series[bool]: ...
    # *** TODO: `fillna` annotation in pandas-stubs is NOT compatible; must `-> Self` ***
    # def fillna(self, value=None, method: FillnaOptions | None = None, inplace: bool = False, **kwargs): ...
    def __contains__(self, other: object) -> bool: ...  # type: ignore[misc]
    @doc(plot_series)
    def plot(self, *args, **kwargs): ...  # type: ignore[override]  # signature of `plot_series` copied in `@doc`
    @doc(_explore_geoseries)  # pyright: ignore[reportUnknownArgumentType]
    def explore(self, *args, **kwargs): ...  # signature of `_explore_geoseries` copied in `@doc`
    def explode(self, ignore_index: bool = False, index_parts: bool = False) -> GeoSeries: ...
    @overload
    def set_crs(
        self, crs: _ConvertibleToCRS, epsg: int | None = None, inplace: bool = False, allow_override: bool = False
    ) -> Self: ...
    @overload
    def set_crs(
        self, crs: _ConvertibleToCRS | None = None, *, epsg: int, inplace: bool = False, allow_override: bool = False
    ) -> Self: ...
    @overload
    def set_crs(self, crs: _ConvertibleToCRS | None, epsg: int, inplace: bool = False, allow_override: bool = False) -> Self: ...
    @overload
    def to_crs(self, crs: _ConvertibleToCRS, epsg: int | None = None) -> GeoSeries: ...
    @overload
    def to_crs(self, crs: _ConvertibleToCRS | None = None, *, epsg: int) -> GeoSeries: ...
    @overload
    def to_crs(self, crs: _ConvertibleToCRS | None, epsg: int) -> GeoSeries: ...
    def estimate_utm_crs(self, datum_name: str = "WGS 84") -> CRS: ...
    def to_json(  # type: ignore[override]
        self,
        show_bbox: bool = True,
        drop_id: bool = False,
        to_wgs84: bool = False,
        *,
        # Keywords from json.dumps
        skipkeys: bool = False,
        ensure_ascii: bool = True,
        check_circular: bool = True,
        allow_nan: bool = True,
        cls: type[json.JSONEncoder] | None = None,
        indent: None | int | str = None,
        separators: tuple[str, str] | None = None,
        default: Callable[..., Any] | None = None,  # as typed in the json stdlib module
        sort_keys: bool = False,
        **kwds,
    ) -> str: ...
    @overload
    def to_wkb(self, hex: Literal[False] = False, **kwargs) -> pd.Series[bytes]: ...
    @overload
    def to_wkb(self, hex: Literal[True], **kwargs) -> pd.Series[str]: ...
    @overload
    def to_wkb(self, hex: bool = False, **kwargs) -> pd.Series[str] | pd.Series[bytes]: ...
    def to_wkt(self, **kwargs) -> pd.Series[str]: ...
    def to_arrow(
        self,
        geometry_encoding: Literal["WKB", "geoarrow", "wkb", "GeoArrow"] = "WKB",
        interleaved: bool | None = True,
        include_z: bool | None = None,
    ) -> GeoArrowArray: ...
    def clip(self, mask: _ClipMask, keep_geom_type: bool = False, sort: bool = False) -> GeoSeries: ...  # type: ignore[override]



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import google.protobuf.wrappers_pb2

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class TrackableObjectGraph(google.protobuf.message.Message):
    """A TensorBundle addition which saves extra information about the objects which
    own variables, allowing for more robust checkpoint loading into modified
    programs.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class TrackableObject(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        @typing.final
        class ObjectReference(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            NODE_ID_FIELD_NUMBER: builtins.int
            LOCAL_NAME_FIELD_NUMBER: builtins.int
            node_id: builtins.int
            """An index into `TrackableObjectGraph.nodes`, indicating the object
            being referenced.
            """
            local_name: builtins.str
            """A user-provided name for the edge."""
            def __init__(self, *, node_id: builtins.int | None = ..., local_name: builtins.str | None = ...) -> None: ...
            def ClearField(self, field_name: typing.Literal["local_name", b"local_name", "node_id", b"node_id"]) -> None: ...

        @typing.final
        class SerializedTensor(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            NAME_FIELD_NUMBER: builtins.int
            FULL_NAME_FIELD_NUMBER: builtins.int
            CHECKPOINT_KEY_FIELD_NUMBER: builtins.int
            name: builtins.str
            """A name for the Tensor. Simple variables have only one
            `SerializedTensor` named "VARIABLE_VALUE" by convention. This value may
            be restored on object creation as an optimization.
            """
            full_name: builtins.str
            """The full name of the variable/tensor, if applicable. Used to allow
            name-based loading of checkpoints which were saved using an
            object-based API. Should match the checkpoint key which would have been
            assigned by tf.train.Saver.
            """
            checkpoint_key: builtins.str
            """The generated name of the Tensor in the checkpoint."""
            def __init__(
                self,
                *,
                name: builtins.str | None = ...,
                full_name: builtins.str | None = ...,
                checkpoint_key: builtins.str | None = ...,
            ) -> None: ...
            def ClearField(
                self, field_name: typing.Literal["checkpoint_key", b"checkpoint_key", "full_name", b"full_name", "name", b"name"]
            ) -> None: ...

        @typing.final
        class SlotVariableReference(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            ORIGINAL_VARIABLE_NODE_ID_FIELD_NUMBER: builtins.int
            SLOT_NAME_FIELD_NUMBER: builtins.int
            SLOT_VARIABLE_NODE_ID_FIELD_NUMBER: builtins.int
            original_variable_node_id: builtins.int
            """An index into `TrackableObjectGraph.nodes`, indicating the
            variable object this slot was created for.
            """
            slot_name: builtins.str
            """The name of the slot (e.g. "m"/"v")."""
            slot_variable_node_id: builtins.int
            """An index into `TrackableObjectGraph.nodes`, indicating the
            `Object` with the value of the slot variable.
            """
            def __init__(
                self,
                *,
                original_variable_node_id: builtins.int | None = ...,
                slot_name: builtins.str | None = ...,
                slot_variable_node_id: builtins.int | None = ...,
            ) -> None: ...
            def ClearField(
                self,
                field_name: typing.Literal[
                    "original_variable_node_id",
                    b"original_variable_node_id",
                    "slot_name",
                    b"slot_name",
                    "slot_variable_node_id",
                    b"slot_variable_node_id",
                ],
            ) -> None: ...

        CHILDREN_FIELD_NUMBER: builtins.int
        ATTRIBUTES_FIELD_NUMBER: builtins.int
        SLOT_VARIABLES_FIELD_NUMBER: builtins.int
        REGISTERED_SAVER_FIELD_NUMBER: builtins.int
        HAS_CHECKPOINT_VALUES_FIELD_NUMBER: builtins.int
        @property
        def children(
            self,
        ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
            global___TrackableObjectGraph.TrackableObject.ObjectReference
        ]:
            """Objects which this object depends on."""

        @property
        def attributes(
            self,
        ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
            global___TrackableObjectGraph.TrackableObject.SerializedTensor
        ]:
            """Serialized data specific to this object."""

        @property
        def slot_variables(
            self,
        ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
            global___TrackableObjectGraph.TrackableObject.SlotVariableReference
        ]:
            """Slot variables owned by this object."""

        @property
        def registered_saver(self) -> global___RegisteredSaver:
            """The registered saver used to save this object. If this saver is not
            present when loading the checkpoint, then loading will fail.
            """

        @property
        def has_checkpoint_values(self) -> google.protobuf.wrappers_pb2.BoolValue:
            """Whether this object has checkpoint values or descendants with checkpoint
            values. This is computed at save time to avoid traversing the entire
            object graph proto when restoring (which also has to traverse the live
            object graph).
            """

        def __init__(
            self,
            *,
            children: collections.abc.Iterable[global___TrackableObjectGraph.TrackableObject.ObjectReference] | None = ...,
            attributes: collections.abc.Iterable[global___TrackableObjectGraph.TrackableObject.SerializedTensor] | None = ...,
            slot_variables: (
                collections.abc.Iterable[global___TrackableObjectGraph.TrackableObject.SlotVariableReference] | None
            ) = ...,
            registered_saver: global___RegisteredSaver | None = ...,
            has_checkpoint_values: google.protobuf.wrappers_pb2.BoolValue | None = ...,
        ) -> None: ...
        def HasField(
            self,
            field_name: typing.Literal[
                "has_checkpoint_values", b"has_checkpoint_values", "registered_saver", b"registered_saver"
            ],
        ) -> builtins.bool: ...
        def ClearField(
            self,
            field_name: typing.Literal[
                "attributes",
                b"attributes",
                "children",
                b"children",
                "has_checkpoint_values",
                b"has_checkpoint_values",
                "registered_saver",
                b"registered_saver",
                "slot_variables",
                b"slot_variables",
            ],
        ) -> None: ...

    NODES_FIELD_NUMBER: builtins.int
    @property
    def nodes(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___TrackableObjectGraph.TrackableObject]: ...
    def __init__(
        self, *, nodes: collections.abc.Iterable[global___TrackableObjectGraph.TrackableObject] | None = ...
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["nodes", b"nodes"]) -> None: ...

global___TrackableObjectGraph = TrackableObjectGraph

@typing.final
class RegisteredSaver(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    OBJECT_NAME_FIELD_NUMBER: builtins.int
    name: builtins.str
    """The name of the registered saver/restore function."""
    object_name: builtins.str
    """Unique auto-generated name of the object."""
    def __init__(self, *, name: builtins.str | None = ..., object_name: builtins.str | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["name", b"name", "object_name", b"object_name"]) -> None: ...

global___RegisteredSaver = RegisteredSaver



import datetime
from _typeshed import Incomplete
from math import _SupportsFloatOrIndex
from typing import Final

hour: Final[float]
minute: Final[float]
second: Final[float]
newton_precision: Final[float]

def calculate_equinoxes(year, timezone: str = "UTC") -> tuple[Incomplete, Incomplete]: ...
def get_current_longitude(current_date, earth, sun): ...
def newton(f, x0, x1, precision=..., **func_kwargs): ...
def newton_angle_function(t, ts, target_angle, body1, body2): ...
def solar_term(year: int, degrees: _SupportsFloatOrIndex, timezone: str = "UTC") -> datetime.date: ...



from authlib.oauth2.rfc6749.authorization_server import AuthorizationServer
from authlib.oauth2.rfc6749.requests import OAuth2Request
from authlib.oauth2.rfc6749.resource_protector import ResourceProtector

from .claims import UserInfo

class UserInfoEndpoint:
    ENDPOINT_NAME: str
    server: AuthorizationServer | None
    resource_protector: ResourceProtector | None
    def __init__(
        self, server: AuthorizationServer | None = None, resource_protector: ResourceProtector | None = None
    ) -> None: ...
    def create_endpoint_request(self, request: OAuth2Request): ...
    def __call__(self, request: OAuth2Request) -> tuple[int, dict[str, str | None], list[tuple[str, str]]]: ...
    def generate_user_info(self, user, scope: str) -> UserInfo: ...
    def get_issuer(self) -> str: ...
    def resolve_private_key(self): ...



from _typeshed import Incomplete
from typing import Final

from reportlab.graphics.widgetbase import Widget

__version__: Final[str]

class PlotArea(Widget):
    x: int
    y: int
    height: int
    width: int
    strokeColor: Incomplete
    strokeWidth: int
    fillColor: Incomplete
    background: Incomplete
    debug: int
    def __init__(self) -> None: ...
    def makeBackground(self): ...



from _typeshed import Incomplete
from collections.abc import Callable, Iterable

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["contracted_edge", "contracted_nodes", "equivalence_classes", "identified_nodes", "quotient_graph"]

@_dispatchable
def equivalence_classes(iterable: Iterable[_Node], relation: Callable[[_Node, _Node], bool]) -> set[frozenset[_Node]]: ...
@_dispatchable
def quotient_graph(
    G: Graph[_Node],
    partition,
    edge_relation=None,
    node_data: Callable[..., Incomplete] | None = None,
    edge_data: Callable[..., Incomplete] | None = None,
    weight: str | None = "weight",
    relabel: bool = False,
    create_using: Graph[_Node] | None = None,
): ...
@_dispatchable
def contracted_nodes(
    G: Graph[_Node], u, v, self_loops: bool = True, copy: bool = True, *, store_contraction_as: str | None = "contraction"
): ...

identified_nodes = contracted_nodes

@_dispatchable
def contracted_edge(
    G: Graph[_Node],
    edge: tuple[Incomplete],
    self_loops: bool = True,
    copy: bool = True,
    *,
    store_contraction_as: str | None = "contraction",
): ...



from ..base import OAuth2Error

__all__ = ["InvalidRequestUriError", "InvalidRequestObjectError", "RequestNotSupportedError", "RequestUriNotSupportedError"]

class InvalidRequestUriError(OAuth2Error):
    error: str
    description: str
    status_code: int

class InvalidRequestObjectError(OAuth2Error):
    error: str
    description: str
    status_code: int

class RequestNotSupportedError(OAuth2Error):
    error: str
    description: str
    status_code: int

class RequestUriNotSupportedError(OAuth2Error):
    error: str
    description: str
    status_code: int



from _typeshed import Incomplete
from typing import Final

__version__: Final[str]

def getStdMonthNames(): ...
def getStdShortMonthNames(): ...
def getStdDayNames(): ...
def getStdShortDayNames(): ...
def isLeapYear(year): ...

class NormalDateException(Exception): ...

class NormalDate:
    def __init__(self, normalDate=None) -> None: ...
    def add(self, days) -> None: ...
    def __add__(self, days): ...
    def __radd__(self, days): ...
    def clone(self): ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def __ge__(self, other): ...
    def __gt__(self, other): ...
    def day(self): ...
    def dayOfWeek(self): ...
    @property
    def __day_of_week_name__(self): ...
    def dayOfWeekAbbrev(self): ...
    def dayOfWeekName(self): ...
    def dayOfYear(self): ...
    def daysBetweenDates(self, normalDate): ...
    def equals(self, target): ...
    def endOfMonth(self): ...
    def firstDayOfMonth(self): ...
    def formatUS(self): ...
    def formatUSCentury(self): ...
    def formatMS(self, fmt): ...
    def __hash__(self): ...
    def __int__(self) -> int: ...
    def isLeapYear(self): ...
    def lastDayOfMonth(self): ...
    def localeFormat(self): ...
    def month(self): ...
    @property
    def __month_name__(self): ...
    def monthAbbrev(self): ...
    def monthName(self): ...
    def normalize(self, scalar) -> None: ...
    def range(self, days): ...
    def scalar(self): ...
    def setDay(self, day) -> None: ...
    def setMonth(self, month) -> None: ...
    normalDate: Incomplete
    def setNormalDate(self, normalDate) -> None: ...
    def setYear(self, year) -> None: ...
    def __sub__(self, v): ...
    def __rsub__(self, v): ...
    def toTuple(self): ...
    def year(self): ...

def bigBang(): ...
def bigCrunch(): ...
def dayOfWeek(y, m, d): ...
def firstDayOfYear(year): ...
def FND(d): ...

Epoch: Incomplete
ND = NormalDate
BDEpoch: Incomplete
BDEpochScalar: int

class BusinessDate(NormalDate):
    def add(self, days) -> None: ...
    def __add__(self, days): ...
    def __sub__(self, v): ...
    def asNormalDate(self): ...
    def daysBetweenDates(self, normalDate): ...
    def normalize(self, i) -> None: ...
    def scalar(self): ...
    def setNormalDate(self, normalDate) -> None: ...
